from . import AWSObject, AWSProperty, Tags


class DomainValidationOption(AWSProperty):
    props = {
        'DomainName': (str, True),
        'ValidationDomain': (str, True),
    }


class Certificate(AWSObject):
    resource_type = "AWS::CertificateManager::Certificate"

    props = {
        'DomainName': (str, True),
        'DomainValidationOptions': ([DomainValidationOption], False),
        'SubjectAlternativeNames': ([str], False),
        'Tags': ((Tags, list), False),
        'ValidationMethod': (str, False),
    }



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (boolean, integer)

VALID_SIGNIN_ALGORITHM = ('SHA256WITHECDSA', 'SHA256WITHRSA',
                          'SHA384WITHECDSA', 'SHA384WITHRSA',
                          'SHA512WITHECDSA', 'SHA512WITHRSA')
VALID_VALIDITY_TYPE = ('ABSOLUTE', 'DAYS', 'END_DATE',
                       'MONTHS', 'YEARS')
VALID_KEY_ALGORITHM = ('EC_prime256v1', 'EC_secp384r1',
                       'RSA_2048', 'RSA_4096')
VALID_CERTIFICATEAUTHORITY_TYPE = ('ROOT', 'SUBORDINATE')


def validate_validity_type(validity_type):
    """Certificate Validity Type validation rule."""
    if validity_type not in VALID_VALIDITY_TYPE:
        raise ValueError("Certificate Validity Type must be one of: %s" %
                         ", ".join(VALID_VALIDITY_TYPE))
    return validity_type


def validate_signing_algorithm(signing_algorithm):
    """Certificate SigningAlgorithm validation rule."""
    if signing_algorithm not in VALID_SIGNIN_ALGORITHM:
        raise ValueError("Certificate SigningAlgorithm must be one of: %s" %
                         ", ".join(VALID_SIGNIN_ALGORITHM))
    return signing_algorithm


def validate_key_algorithm(key_algorithm):
    """CertificateAuthority KeyAlgorithm validation rule."""
    if key_algorithm not in VALID_KEY_ALGORITHM:
        raise ValueError("CertificateAuthority KeyAlgorithm must be one of: %s" %  # NOQA
                         ", ".join(VALID_KEY_ALGORITHM))
    return key_algorithm


def validate_certificateauthority_type(certificateauthority_type):
    """CertificateAuthority Type validation rule."""
    if certificateauthority_type not in VALID_CERTIFICATEAUTHORITY_TYPE:
        raise ValueError("CertificateAuthority Type must be one of: %s" %
                         ", ".join(VALID_CERTIFICATEAUTHORITY_TYPE))
    return certificateauthority_type


class Validity(AWSProperty):
    props = {
        'Type': (validate_validity_type, True),
        'Value': (integer, True),
    }


class Certificate(AWSProperty):
    resource_type = "AWS::ACMPCA::Certificate"

    props = {
        'CertificateAuthorityArn': (str, True),
        'CertificateSigningRequest': (str, True),
        'SigningAlgorithm': (validate_signing_algorithm, True),
        'TemplateArn': (str, False),
        'Validity': (Validity, True),
    }


class CertificateAuthorityActivation(AWSObject):
    resource_type = "AWS::ACMPCA::CertificateAuthorityActivation"

    props = {
        'Certificate': (str, True),
        'CertificateAuthorityArn': (str, True),
        'CertificateChain': (str, False),
        'Status': (str, False),
    }


class CrlConfiguration(AWSProperty):
    props = {
        'CustomCname': (str, False),
        'Enabled': (boolean, False),
        'ExpirationInDays': (integer, False),
        'S3BucketName': (str, False),
    }


class RevocationConfiguration(AWSProperty):
    props = {
        'CrlConfiguration': (CrlConfiguration, False)
    }


class Subject(AWSObject):
    props = {
        'CommonName': (str, False),
        'Country': (str, False),
        'DistinguishedNameQualifier': (str, False),
        'GenerationQualifier': (str, False),
        'GivenName': (str, False),
        'Initials': (str, False),
        'Locality': (str, False),
        'Organization': (str, False),
        'OrganizationalUnit': (str, False),
        'Pseudonym': (str, False),
        'SerialNumber': (str, False),
        'State': (str, False),
        'Surname': (str, False),
        'Title': (str, False),
    }


class CertificateAuthority(AWSObject):
    resource_type = "AWS::ACMPCA::CertificateAuthority"

    props = {
        'KeyAlgorithm': (validate_key_algorithm, True),
        'RevocationConfiguration': (RevocationConfiguration, False),
        'SigningAlgorithm': (validate_signing_algorithm, True),
        'Subject': (Subject, True),
        'Tags': (Tags, False),
        'Type': (validate_certificateauthority_type, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from . import AWSObject
from . import AWSProperty
from troposphere import Tags
from .validators import boolean


class Activity(AWSObject):
    resource_type = "AWS::StepFunctions::Activity"

    props = {
        'Name': (str, True),
        'Tags': (Tags, False),
    }


class CloudWatchLogsLogGroup(AWSProperty):
    props = {
        'LogGroupArn': (str, True),
    }


class LogDestination(AWSProperty):
    props = {
        'CloudWatchLogsLogGroup': (CloudWatchLogsLogGroup, False),
    }


class LoggingConfiguration(AWSProperty):
    props = {
        'Destinations': ([LogDestination], False),
        'IncludeExecutionData': (boolean, False),
        'Level': (str, False),
    }


class StateMachine(AWSObject):
    resource_type = "AWS::StepFunctions::StateMachine"

    props = {
        'DefinitionString': (str, True),
        'LoggingConfiguration': (LoggingConfiguration, False),
        'RoleArn': (str, True),
        'StateMachineName': (str, False),
        'StateMachineType': (str, False),
        'Tags': (Tags, False),
    }



# Copyright (c) 2016, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .validators import boolean, integer, positive_integer


VALID_IMAGE_PULL_CREDENTIALS = ('CODEBUILD', 'SERVICE_ROLE')
VALID_CREDENTIAL_PROVIDERS = ('SECRETS_MANAGER')
VALID_WEBHOOKFILTER_TYPES = ('EVENT', 'ACTOR_ACCOUNT_ID', 'HEAD_REF',
                             'BASE_REF', 'FILE_PATH')
VALID_PROJECTFILESYSTEMLOCATION_TYPE = ('EFS')


def validate_image_pull_credentials(image_pull_credentials):
    """Validate ImagePullCredentialsType for Project"""

    if image_pull_credentials not in VALID_IMAGE_PULL_CREDENTIALS:
        raise ValueError("Project ImagePullCredentialsType must be one of: %s" %  # NOQA
                         ", ".join(VALID_IMAGE_PULL_CREDENTIALS))
    return image_pull_credentials


def validate_credentials_provider(credential_provider):
    """Validate CredentialProvider for Project's RegistryCredential"""

    if credential_provider not in VALID_CREDENTIAL_PROVIDERS:
        raise ValueError("RegistryCredential CredentialProvider must be one of: %s" %  # NOQA
                         ", ".join(VALID_CREDENTIAL_PROVIDERS))
    return credential_provider


def validate_webhookfilter_type(webhookfilter_type):
    """Validate WebHookFilter type property for a Project"""

    if webhookfilter_type not in VALID_WEBHOOKFILTER_TYPES:
        raise ValueError("Project Webhookfilter Type must be one of: %s" %
                         ", ".join(VALID_WEBHOOKFILTER_TYPES))
    return webhookfilter_type


def validate_projectfilesystemlocation_type(projectfilesystemlocation_type):
    """Validate ProjectFileSystemLocation type property"""

    if projectfilesystemlocation_type not in VALID_PROJECTFILESYSTEMLOCATION_TYPE:  # NOQA
        raise ValueError("ProjectFileSystemLocation Type must be one of: %s" %  # NOQA
                         ", ".join(VALID_PROJECTFILESYSTEMLOCATION_TYPE))
    return projectfilesystemlocation_type


class SourceAuth(AWSProperty):
    props = {
        'Resource': (str, False),
        'Type': (str, True),
    }

    def validate(self):
        valid_types = [
            'OAUTH'
        ]
        auth_types = self.properties.get('Type')
        if auth_types not in valid_types:
            raise ValueError('SourceAuth Type: must be one of %s' %
                             ','.join(valid_types))


class Artifacts(AWSProperty):
    props = {
        'ArtifactIdentifier': (str, False),
        'EncryptionDisabled': (boolean, False),
        'Location': (str, False),
        'Name': (str, False),
        'NamespaceType': (str, False),
        'OverrideArtifactName': (boolean, False),
        'Packaging': (str, False),
        'Path': (str, False),
        'Type': (str, True),
    }

    def validate(self):
        valid_types = [
            'CODEPIPELINE',
            'NO_ARTIFACTS',
            'S3',
        ]
        artifact_type = self.properties.get('Type')
        if artifact_type not in valid_types:
            raise ValueError('Artifacts Type: must be one of %s' %
                             ','.join(valid_types))

        if artifact_type == 'S3':
            for required_property in ['Name', 'Location']:
                if not self.properties.get(required_property):
                    raise ValueError(
                        'Artifacts Type S3: requires %s to be set' %
                        required_property
                    )


class EnvironmentVariable(AWSProperty):
    props = {
        'Name': (str, True),
        'Type': (str, False),
        'Value': (str, True),
    }

    def validate(self):
        if 'Type' in self.properties:
            valid_types = [
                'PARAMETER_STORE',
                'PLAINTEXT',
            ]
            env_type = self.properties.get('Type')
            if env_type not in valid_types:
                raise ValueError(
                    'EnvironmentVariable Type: must be one of %s' %
                    ','.join(valid_types))


class RegistryCredential(AWSProperty):
    props = {
        'Credential': (str, True),
        'CredentialProvider': (validate_credentials_provider, True),
    }


class Environment(AWSProperty):
    props = {
        'Certificate': (str, False),
        'ComputeType': (str, True),
        'EnvironmentVariables': ((list, [EnvironmentVariable]), False),
        'Image': (str, True),
        'ImagePullCredentialsType': (validate_image_pull_credentials, False),
        'PrivilegedMode': (boolean, False),
        'RegistryCredential': (RegistryCredential, False),
        'Type': (str, True),
    }

    def validate(self):
        valid_types = [
            'LINUX_CONTAINER',
            'WINDOWS_CONTAINER',
        ]
        env_type = self.properties.get('Type')
        if env_type not in valid_types:
            raise ValueError('Environment Type: must be one of %s' %
                             ','.join(valid_types))


class ProjectCache(AWSProperty):
    props = {
        'Location': (str, False),
        'Modes': ([str], False),
        'Type': (str, True),
    }

    def validate(self):
        valid_types = [
            'NO_CACHE',
            'LOCAL',
            'S3',
        ]
        cache_type = self.properties.get('Type')
        if cache_type not in valid_types:
            raise ValueError('ProjectCache Type: must be one of %s' %
                             ','.join(valid_types))


class GitSubmodulesConfig(AWSProperty):
    props = {
        'FetchSubmodules': (boolean, True),
    }


class Source(AWSProperty):
    props = {
        'Auth': (SourceAuth, False),
        'BuildSpec': (str, False),
        'GitCloneDepth': (positive_integer, False),
        'GitSubmodulesConfig': (GitSubmodulesConfig, False),
        'InsecureSsl': (boolean, False),
        'Location': (str, False),
        'ReportBuildStatus': (boolean, False),
        'SourceIdentifier': (str, False),
        'Type': (str, True),
    }

    def validate(self):
        valid_types = [
            'BITBUCKET',
            'CODECOMMIT',
            'CODEPIPELINE',
            'GITHUB',
            'GITHUB_ENTERPRISE',
            'NO_SOURCE',
            'S3',
        ]

        location_agnostic_types = [
            'CODEPIPELINE',
            'NO_SOURCE',
        ]

        source_type = self.properties.get('Type')

        # Don't do additional checks if source_type can't
        # be determined (for example, being a Ref).
        if isinstance(source_type, AWSHelperFn):
            return

        if source_type not in valid_types:
            raise ValueError('Source Type: must be one of %s' %
                             ','.join(valid_types))

        location = self.properties.get('Location')

        if source_type not in location_agnostic_types and not location:
            raise ValueError(
                'Source Location: must be defined when type is %s' %
                source_type
            )

        auth = self.properties.get('Auth')
        if auth is not None and source_type != 'GITHUB':
            raise ValueError("SourceAuth: must only be defined when using "
                             "'GITHUB' Source Type.")


class VpcConfig(AWSProperty):
    props = {
        'SecurityGroupIds': ([str], True),
        'Subnets': ([str], True),
        'VpcId': (str, True),
    }


class WebhookFilter(AWSProperty):
    props = {
        'ExcludeMatchedPattern': (boolean, False),
        'Pattern': (str, True),
        'Type': (validate_webhookfilter_type, True),
    }


class ProjectTriggers(AWSProperty):
    props = {
        'Webhook': (boolean, False),
        'FilterGroups': (list, False)
    }

    def validate(self):
        """ FilterGroups, if set, needs to be a list of a list of
        WebhookFilters
        """
        filter_groups = self.properties.get('FilterGroups')
        if filter_groups is not None:
            if not isinstance(filter_groups, list):
                self._raise_type('FilterGroups', filter_groups, list)

            for counti, elem in enumerate(filter_groups):
                if not isinstance(elem, list):
                    self._raise_type(
                        'FilterGroups[{}]'.format(counti),
                        filter_groups[counti], list
                    )
                for countj, hook in enumerate(filter_groups[counti]):
                    if not isinstance(hook, WebhookFilter):
                        self._raise_type(
                            'FilterGroups[{}][{}]'.format(counti, countj),
                            hook, WebhookFilter
                        )


def validate_status(status):
    """ Validate status
    :param status: The Status of CloudWatchLogs or S3Logs
    :return: The provided value if valid
    """
    valid_statuses = [
        'ENABLED',
        'DISABLED'
    ]

    if status not in valid_statuses:
        raise ValueError('Status: must be one of %s' %
                         ','.join(valid_statuses))
    return status


class CloudWatchLogs(AWSProperty):
    props = {
        "Status": (validate_status, True),
        "GroupName": (str, False),
        "StreamName": (str, False)
    }


class S3Logs(AWSProperty):
    props = {
        "EncryptionDisabled": (boolean, False),
        "Status": (validate_status, True),
        "Location": (str, False)
    }


class LogsConfig(AWSProperty):
    props = {
        'CloudWatchLogs': (CloudWatchLogs, False),
        'S3Logs': (S3Logs, False)
    }


class ProjectSourceVersion(AWSProperty):
    props = {
        'SourceIdentifier': (str, True),
        'SourceVersion': (str, False),
    }


class ProjectFileSystemLocation(AWSProperty):
    props = {
        'Identifier': (str, True),
        'Location': (str, True),
        'MountOptions': (str, False),
        'MountPoint': (str, True),
        'Type': (validate_projectfilesystemlocation_type, True),
    }


class Project(AWSObject):
    resource_type = "AWS::CodeBuild::Project"

    props = {
        'Artifacts': (Artifacts, True),
        'BadgeEnabled': (boolean, False),
        'Cache': (ProjectCache, False),
        'Description': (str, False),
        'EncryptionKey': (str, False),
        'Environment': (Environment, True),
        'FileSystemLocations': ([ProjectFileSystemLocation], False),
        'LogsConfig': (LogsConfig, False),
        'Name': (str, False),
        'QueuedTimeoutInMinutes': (integer, False),
        'SecondaryArtifacts': ([Artifacts], False),
        'SecondarySourceVersions': ([ProjectSourceVersion], False),
        'SecondarySources': ([Source], False),
        'ServiceRole': (str, True),
        'Source': (Source, True),
        'SourceVersion': (str, False),
        'Tags': (Tags, False),
        'TimeoutInMinutes': (integer, False),
        'Triggers': (ProjectTriggers, False),
        'VpcConfig': (VpcConfig, False),
    }


class S3ReportExportConfig(AWSProperty):
    props = {
        'Bucket': (str, True),
        'EncryptionDisabled': (boolean, False),
        'EncryptionKey': (str, False),
        'Packaging': (str, False),
        'Path': (str, False),
    }


class ReportExportConfig(AWSProperty):
    props = {
        'ExportConfigType': (str, True),
        'S3Destination': (S3ReportExportConfig, False),
    }


class ReportGroup(AWSObject):
    resource_type = "AWS::CodeBuild::ReportGroup"

    props = {
        'ExportConfig': (ReportExportConfig, True),
        'Name': (str, False),
        'Type': (str, True),
    }


class SourceCredential(AWSObject):
    resource_type = "AWS::CodeBuild::SourceCredential"

    props = {
        'AuthType': (str, True),
        'ServerType': (str, True),
        'Token': (str, True),
        'Username': (str, False),
    }



from . import AWSObject, AWSProperty
from .validators import boolean, exactly_one, integer, positive_integer


class LaunchTemplateSpecification(AWSProperty):
    props = {
        "LaunchTemplateId": (str, False),
        "LaunchTemplateName": (str, False),
        "Version": (str, False),
    }

    def validate(self):
        template_ids = [
            'LaunchTemplateId',
            'LaunchTemplateName'
        ]
        exactly_one(self.__class__.__name__, self.properties, template_ids)


def validate_allocation_strategy(allocation_strategy):
    """ Validate allocation strategy
    :param allocation_strategy: Allocation strategy for ComputeResource
    :return: The provided value if valid
    """
    valid_strategies = [
        "BEST_FIT",
        "BEST_FIT_PROGRESSIVE",
        "SPOT_CAPACITY_OPTIMIZED"
    ]
    if allocation_strategy not in valid_strategies:
        raise ValueError(
            "{} is not a valid strategy".format(allocation_strategy)
        )
    return allocation_strategy


class ComputeResources(AWSProperty):

    props = {
        "AllocationStrategy": (validate_allocation_strategy, False),
        "SpotIamFleetRole": (str, False),
        "MaxvCpus": (positive_integer, True),
        "SecurityGroupIds": ([str], True),
        "BidPercentage": (positive_integer, False),
        "Type": (str, True),
        "Subnets": ([str], True),
        "MinvCpus": (positive_integer, True),
        "LaunchTemplate": (LaunchTemplateSpecification, False),
        "ImageId": (str, False),
        "InstanceRole": (str, True),
        "InstanceTypes": ([str], True),
        "Ec2KeyPair": (str, False),
        "PlacementGroup": (str, False),
        "Tags": (dict, False),
        "DesiredvCpus": (positive_integer, False)
    }


class Device(AWSProperty):
    props = {
        'ContainerPath': (str, False),
        'HostPath': (str, False),
        'Permissions': ([str], False),
    }


class LinuxParameters(AWSProperty):
    props = {
        'Devices': ([Device], False),
    }


class MountPoints(AWSProperty):

    props = {
        "ReadOnly": (bool, False),
        "SourceVolume": (str, False),
        "ContainerPath": (str, False)
    }


class VolumesHost(AWSProperty):

    props = {
        "SourcePath": (str, False)
    }


class Volumes(AWSProperty):

    props = {
        "Host": (VolumesHost, False),
        "Name": (str, False)
    }


class Environment(AWSProperty):

    props = {
        "Value": (str, False),
        "Name": (str, False)
    }


class ResourceRequirement(AWSProperty):
    props = {
        'Type': (str, False),
        'Value': (str, False),
    }


class Ulimit(AWSProperty):

    props = {
        "SoftLimit": (positive_integer, True),
        "HardLimit": (positive_integer, True),
        "Name": (str, True)
    }


class ContainerProperties(AWSProperty):

    props = {
        'Command': ([str], False),
        'Environment': ([Environment], False),
        'Image': (str, True),
        'InstanceType': (str, False),
        'JobRoleArn': (str, False),
        'LinuxParameters': (LinuxParameters, False),
        'Memory': (positive_integer, True),
        'MountPoints': ([MountPoints], False),
        'Privileged': (boolean, False),
        'ReadonlyRootFilesystem': (boolean, False),
        'ResourceRequirements': ([ResourceRequirement], False),
        'Ulimits': ([Ulimit], False),
        'User': (str, False),
        'Vcpus': (positive_integer, True),
        'Volumes': ([Volumes], False),
    }


class RetryStrategy(AWSProperty):

    props = {
        "Attempts": (positive_integer, False)
    }


class Timeout(AWSProperty):
    props = {
        'AttemptDurationSeconds': (integer, False),
    }


class JobDefinition(AWSObject):
    resource_type = "AWS::Batch::JobDefinition"

    props = {
        'ContainerProperties': (ContainerProperties, True),
        'JobDefinitionName': (str, False),
        'Parameters': (dict, False),
        'RetryStrategy': (RetryStrategy, False),
        'Timeout': (Timeout, False),
        'Type': (str, True),
    }


def validate_environment_state(environment_state):
    """ Validate response type
    :param environment_state: State of the environment
    :return: The provided value if valid
    """
    valid_states = [
        "ENABLED",
        "DISABLED"
    ]
    if environment_state not in valid_states:
        raise ValueError(
            "{} is not a valid environment state".format(environment_state)
        )
    return environment_state


class ComputeEnvironment(AWSObject):
    resource_type = "AWS::Batch::ComputeEnvironment"

    props = {
        "Type": (str, True),
        "ServiceRole": (str, True),
        "ComputeEnvironmentName": (str, False),
        "ComputeResources": (ComputeResources, True),
        "State": (validate_environment_state, False)
    }


class ComputeEnvironmentOrder(AWSProperty):

    props = {
        "ComputeEnvironment": (str, True),
        "Order": (positive_integer, True)
    }


def validate_queue_state(queue_state):
    """ Validate response type
    :param queue_state: State of the queue
    :return: The provided value if valid
    """
    valid_states = [
        "ENABLED",
        "DISABLED"
    ]
    if queue_state not in valid_states:
        raise ValueError(
            "{} is not a valid queue state".format(queue_state)
        )
    return queue_state


class JobQueue(AWSObject):
    resource_type = "AWS::Batch::JobQueue"

    props = {
        "ComputeEnvironmentOrder": ([ComputeEnvironmentOrder], True),
        "Priority": (positive_integer, True),
        "State": (validate_queue_state, False),
        "JobQueueName": (str, False)
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, exactly_one, mutually_exclusive,\
    positive_integer

KEY_ONLY = "KEY_ONLY"
VALUE_ONLY = "VALUE_ONLY"
KEY_AND_VALUE = "KEY_AND_VALUE"


class GitHubLocation(AWSProperty):
    props = {
        'CommitId': (str, True),
        'Repository': (str, True),
    }


class S3Location(AWSProperty):
    props = {
        'Bucket': (str, True),
        'BundleType': (str, True),
        'ETag': (str, False),
        'Key': (str, True),
        'Version': (str, False),
    }


class Revision(AWSProperty):
    props = {
        'GitHubLocation': (GitHubLocation, False),
        'RevisionType': (str, False),
        'S3Location': (S3Location, False),
    }


def deployment_option_validator(x):
    valid_values = ['WITH_TRAFFIC_CONTROL', 'WITHOUT_TRAFFIC_CONTROL']
    if x not in valid_values:
        raise ValueError("Deployment Option value must be one of: %s" %
                         ', '.join(valid_values))
    return x


def deployment_type_validator(x):
    valid_values = ['IN_PLACE', 'BLUE_GREEN']
    if x not in valid_values:
        raise ValueError("Deployment Type value must be one of: %s" %
                         ', '.join(valid_values))
    return x


class AutoRollbackConfiguration(AWSProperty):
    props = {
        'Enabled': (bool, False),
        'Events': ([str], False)
    }


class Deployment(AWSProperty):
    props = {
        'Description': (str, False),
        'IgnoreApplicationStopFailures': (bool, False),
        'Revision': (Revision, True),
    }


class DeploymentStyle(AWSProperty):
    props = {
        'DeploymentOption': (deployment_option_validator, False),
        'DeploymentType': (deployment_type_validator, False),
    }


class Ec2TagFilters(AWSProperty):
    props = {
        'Key': (str, False),
        'Type': (str, True),
        'Value': (str, False),
    }


class TagFilters(AWSProperty):
    props = {
        'Key': (str, False),
        'Type': (str, False),
        'Value': (str, False)
    }


class ElbInfoList(AWSProperty):
    props = {
        'Name': (str, False)
    }


class TargetGroupInfoList(AWSProperty):
    props = {
        'Name': (str, False)
    }


class LoadBalancerInfo(AWSProperty):
    props = {
        'ElbInfoList': ([ElbInfoList], False),
        'TargetGroupInfoList': ([TargetGroupInfoList], False),
    }

    def validate(self):
        conds = [
            'ElbInfoList',
            'TargetGroupInfoList'
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class OnPremisesInstanceTagFilters(AWSProperty):
    props = {
        'Key': (str, False),
        'Type': (str, False),
        'Value': (str, False),
    }


class MinimumHealthyHosts(AWSProperty):
    props = {
        'Type': (str, False),
        'Value': (positive_integer, False),
    }


class Application(AWSObject):
    resource_type = "AWS::CodeDeploy::Application"

    props = {
        'ApplicationName': (str, False),
        'ComputePlatform': (str, False),
    }


class DeploymentConfig(AWSObject):
    resource_type = "AWS::CodeDeploy::DeploymentConfig"

    props = {
        'DeploymentConfigName': (str, False),
        'MinimumHealthyHosts': (MinimumHealthyHosts, False),
    }


class Alarm(AWSProperty):
    props = {
        'Name': (str, False),
    }


class AlarmConfiguration(AWSProperty):
    props = {
        'Alarms': ([Alarm], False),
        'Enabled': (boolean, False),
        'IgnorePollAlarmFailure': (boolean, False),
    }


class TriggerConfig(AWSProperty):
    props = {
        'TriggerEvents': ([str], False),
        'TriggerName': (str, False),
        'TriggerTargetArn': (str, False),
    }


class Ec2TagSetListObject(AWSProperty):
    props = {
        'Ec2TagGroup': ([Ec2TagFilters], False)
    }


class Ec2TagSet(AWSProperty):
    props = {
        'Ec2TagSetList': ([Ec2TagSetListObject], False)
    }


class OnPremisesTagSetObject(AWSProperty):
    props = {
        'OnPremisesTagGroup': ([TagFilters], False)
    }


class OnPremisesTagSetList(AWSProperty):
    props = {
        'OnPremisesTagSetList': ([OnPremisesTagSetObject], False)
    }


class OnPremisesTagSet(AWSProperty):
    props = {
        'OnPremisesTagSetList': (OnPremisesTagSetList, False)
    }


class DeploymentGroup(AWSObject):
    resource_type = "AWS::CodeDeploy::DeploymentGroup"

    props = {
        'AlarmConfiguration': (AlarmConfiguration, False),
        'ApplicationName': (str, True),
        'AutoRollbackConfiguration': (AutoRollbackConfiguration, False),
        'AutoScalingGroups': ([str], False),
        'Deployment': (Deployment, False),
        'DeploymentConfigName': (str, False),
        'DeploymentGroupName': (str, False),
        'DeploymentStyle': (DeploymentStyle, False),
        'Ec2TagFilters': ([Ec2TagFilters], False),
        'Ec2TagSet': (Ec2TagSet, False),
        'LoadBalancerInfo': (LoadBalancerInfo, False),
        'OnPremisesInstanceTagFilters': (
            [OnPremisesInstanceTagFilters], False
        ),
        'OnPremisesInstanceTagSet': (OnPremisesTagSet, False),
        'ServiceRoleArn': (str, True),
        'TriggerConfigurations': ([TriggerConfig], False),
    }

    def validate(self):
        ec2_conds = [
            'EC2TagFilters',
            'Ec2TagSet'
        ]
        onPremises_conds = [
            'OnPremisesInstanceTagFilters',
            'OnPremisesInstanceTagSet'
        ]
        mutually_exclusive(self.__class__.__name__,
                           self.properties, ec2_conds)
        mutually_exclusive(self.__class__.__name__,
                           self.properties, onPremises_conds)



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class ConfigurationId(AWSProperty):
    props = {
        'Id': (str, True),
        'Revision': (integer, True),
    }


class EncryptionOptions(AWSProperty):
    props = {
        'KmsKeyId': (str, False),
        'UseAwsOwnedKey': (boolean, True),
    }


class MaintenanceWindow(AWSProperty):
    props = {
        'DayOfWeek': (str, True),
        'TimeOfDay': (str, True),
        'TimeZone': (str, True),
    }


class User(AWSProperty):
    props = {
        'ConsoleAccess': (boolean, False),
        'Groups': ([str], False),
        'Password': (str, True),
        'Username': (str, True),
    }


class LogsConfiguration(AWSProperty):
    props = {
        'Audit': (boolean, False),
        'General': (boolean, False),
    }


class Broker(AWSObject):
    resource_type = "AWS::AmazonMQ::Broker"

    props = {
        'AutoMinorVersionUpgrade': (boolean, True),
        'BrokerName': (str, True),
        'Users': ([User], True),
        'Configuration': (ConfigurationId, False),
        'DeploymentMode': (str, True),
        'EncryptionOptions': (EncryptionOptions, False),
        'EngineType': (str, True),
        'EngineVersion': (str, True),
        'HostInstanceType': (str, True),
        'Logs': (LogsConfiguration, False),
        'MaintenanceWindowStartTime': (MaintenanceWindow, False),
        'PubliclyAccessible': (boolean, True),
        'SecurityGroups': ([str], False),
        'SubnetIds': ([str], False),
        'Tags': ((Tags, list), False),
    }


class Configuration(AWSObject):
    resource_type = "AWS::AmazonMQ::Configuration"

    props = {
        'Data': (str, True),
        'Description': (str, False),
        'EngineType': (str, True),
        'EngineVersion': (str, True),
        'Name': (str, True),
    }


class ConfigurationAssociation(AWSObject):
    resource_type = "AWS::AmazonMQ::ConfigurationAssociation"

    props = {
        'Broker': (str, True),
        'Configuration': (ConfigurationId, True),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, double
from .validators import integer, integer_range, positive_integer


class CsvClassifier(AWSProperty):
    props = {
        'AllowSingleColumn': (boolean, False),
        'ContainsHeader': (str, False),
        'Delimiter': (str, False),
        'DisableValueTrimming': (boolean, False),
        'Header': ([str], False),
        'Name': (str, False),
        'QuoteSymbol': (str, False),
    }


class GrokClassifier(AWSProperty):
    props = {
        'Classification': (str, True),
        'CustomPatterns': (str, False),
        'GrokPattern': (str, True),
        'Name': (str, False),
    }


class JsonClassifier(AWSProperty):
    props = {
        'JsonPath': (str, True),
        'Name': (str, False),
    }


class XMLClassifier(AWSProperty):
    props = {
        'Classification': (str, True),
        'Name': (str, False),
        'RowTag': (str, True),
    }


class Classifier(AWSObject):
    resource_type = "AWS::Glue::Classifier"

    props = {
        'CsvClassifier': (CsvClassifier, False),
        'GrokClassifier': (GrokClassifier, False),
        'JsonClassifier': (JsonClassifier, False),
        'XMLClassifier': (XMLClassifier, False),
    }


class PhysicalConnectionRequirements(AWSProperty):
    props = {
        'AvailabilityZone': (str, False),
        'SecurityGroupIdList': ([str], False),
        'SubnetId': (str, False),
    }


def connection_type_validator(type):
    valid_types = [
        'JDBC',
        'SFTP',
    ]
    if type not in valid_types:
        raise ValueError('% is not a valid value for ConnectionType' % type)
    return type


class ConnectionInput(AWSProperty):
    props = {
        'ConnectionProperties': (dict, True),
        'ConnectionType': (connection_type_validator, True),
        'Description': (str, False),
        'MatchCriteria': ([str], False),
        'Name': (str, False),
        'PhysicalConnectionRequirements':
            (PhysicalConnectionRequirements, False),
    }


class Connection(AWSObject):
    resource_type = "AWS::Glue::Connection"

    props = {
        'CatalogId': (str, True),
        'ConnectionInput': (ConnectionInput, True),
    }


class Schedule(AWSProperty):
    props = {
        'ScheduleExpression': (str, False),
    }


def delete_behavior_validator(value):
    valid_values = [
        'LOG',
        'DELETE_FROM_DATABASE',
        'DEPRECATE_IN_DATABASE',
    ]
    if value not in valid_values:
        raise ValueError('% is not a valid value for DeleteBehavior' % value)
    return value


def update_behavior_validator(value):
    valid_values = [
        'LOG',
        'UPDATE_IN_DATABASE',
    ]
    if value not in valid_values:
        raise ValueError('% is not a valid value for UpdateBehavior' % value)
    return value


class SchemaChangePolicy(AWSProperty):
    props = {
        'DeleteBehavior': (delete_behavior_validator, False),
        'UpdateBehavior': (update_behavior_validator, False),
    }


class CatalogTarget(AWSProperty):
    props = {
        'DatabaseName': (str, False),
        'Tables': ([str], False),
    }


class DynamoDBTarget(AWSProperty):
    props = {
        'Path': (str, False),
    }


class JdbcTarget(AWSProperty):
    props = {
        'ConnectionName': (str, False),
        'Exclusions': ([str], False),
        'Path': (str, False),
    }


class S3Target(AWSProperty):
    props = {
        'Exclusions': ([str], False),
        'Path': (str, False),
    }


class Targets(AWSProperty):
    props = {
        'CatalogTargets': ([CatalogTarget], False),
        'DynamoDBTargets': ([DynamoDBTarget], False),
        'JdbcTargets': ([JdbcTarget], False),
        'S3Targets': ([S3Target], False),
    }


class Crawler(AWSObject):
    resource_type = "AWS::Glue::Crawler"

    props = {
        'Classifiers': ([str], False),
        'Configuration': (str, False),
        'CrawlerSecurityConfiguration': (str, False),
        'DatabaseName': (str, True),
        'Description': (str, False),
        'Name': (str, False),
        'Role': (str, True),
        'Schedule': (Schedule, False),
        'SchemaChangePolicy': (SchemaChangePolicy, False),
        'TablePrefix': (str, False),
        'Tags': (dict, False),
        'Targets': (Targets, True),
    }


class ConnectionPasswordEncryption(AWSProperty):
    props = {
        'KmsKeyId': (str, False),
        'ReturnConnectionPasswordEncrypted': (boolean, False),
    }


class EncryptionAtRest(AWSProperty):
    props = {
        'CatalogEncryptionMode': (str, False),
        'SseAwsKmsKeyId': (str, False),
    }


class DataCatalogEncryptionSettingsProperty(AWSProperty):
    props = {
        'ConnectionPasswordEncryption':
            (ConnectionPasswordEncryption, False),
        'EncryptionAtRest': (EncryptionAtRest, False),
    }


class DataCatalogEncryptionSettings(AWSObject):
    resource_type = "AWS::Glue::DataCatalogEncryptionSettings"

    props = {
        'CatalogId': (str, True),
        'DataCatalogEncryptionSettings':
            (DataCatalogEncryptionSettingsProperty, True),
    }


class DatabaseInput(AWSProperty):
    props = {
        'Description': (str, False),
        'LocationUri': (str, False),
        'Name': (str, False),
        'Parameters': (dict, False),
    }


class Database(AWSObject):
    resource_type = "AWS::Glue::Database"

    props = {
        'CatalogId': (str, True),
        'DatabaseInput': (DatabaseInput, True),
    }


class DevEndpoint(AWSObject):
    resource_type = "AWS::Glue::DevEndpoint"

    props = {
        'Arguments': (dict, False),
        'EndpointName': (str, False),
        'ExtraJarsS3Path': (str, False),
        'ExtraPythonLibsS3Path': (str, False),
        'GlueVersion': (str, False),
        'NumberOfNodes': (integer, False),
        'NumberOfWorkers': (integer, False),
        'PublicKey': (str, False),
        'RoleArn': (str, True),
        'SecurityConfiguration': (str, False),
        'SecurityGroupIds': ([str], False),
        'SubnetId': (str, False),
        'Tags': (dict, False),
        'WorkerType': (str, False),
    }


class ConnectionsList(AWSProperty):
    props = {
        'Connections': ([str], False),
    }


class ExecutionProperty(AWSProperty):
    props = {
        'MaxConcurrentRuns': (positive_integer, False),
    }


class JobCommand(AWSProperty):
    props = {
        'Name': (str, False),
        'PythonVersion': (str, False),
        'ScriptLocation': (str, False),
    }


class NotificationProperty(AWSProperty):
    props = {
        'NotifyDelayAfter': (integer, False),
    }


class Job(AWSObject):
    resource_type = "AWS::Glue::Job"

    props = {
        'AllocatedCapacity': (double, False),
        'Command': (JobCommand, True),
        'Connections': (ConnectionsList, False),
        'DefaultArguments': (dict, False),
        'Description': (str, False),
        'ExecutionProperty': (ExecutionProperty, False),
        'GlueVersion': (str, False),
        'LogUri': (str, False),
        'MaxCapacity': (double, False),
        'MaxRetries': (double, False),
        'Name': (str, False),
        'NotificationProperty': (NotificationProperty, False),
        'NumberOfWorkers': (integer, False),
        'Role': (str, True),
        'SecurityConfiguration': (str, False),
        'Tags': (dict, False),
        'Timeout': (integer, False),
        'WorkerType': (str, False),
    }


class GlueTables(AWSProperty):
    props = {
        'CatalogId': (str, False),
        'ConnectionName': (str, False),
        'DatabaseName': (str, True),
        'TableName': (str, True),
    }


class InputRecordTables(AWSProperty):
    props = {
        'GlueTables': ([GlueTables], False),
    }


class FindMatchesParameters(AWSProperty):
    props = {
        'AccuracyCostTradeoff': (float, False),
        'EnforceProvidedLabels': (boolean, False),
        'PrecisionRecallTradeoff': (float, False),
        'PrimaryKeyColumnName': (str, True),
    }


class TransformParameters(AWSProperty):
    props = {
        'FindMatchesParameters': (FindMatchesParameters, False),
        'TransformType': (str, True),
    }


class MLTransform(AWSObject):
    resource_type = "AWS::Glue::MLTransform"

    props = {
        'Description': (str, False),
        'GlueVersion': (str, False),
        'InputRecordTables': (InputRecordTables, True),
        'MaxCapacity': (double, False),
        'MaxRetries': (integer, False),
        'Name': (str, False),
        'NumberOfWorkers': (integer, False),
        'Role': (str, True),
        'Timeout': (integer, False),
        'TransformParameters': (TransformParameters, True),
        'WorkerType': (str, False),
    }


class Column(AWSProperty):
    props = {
        'Comment': (str, False),
        'Name': (str, True),
        'Type': (str, False),
    }


class Order(AWSProperty):
    props = {
        'Column': (str, True),
        'SortOrder': (integer_range(0, 1), False),
    }


class SerdeInfo(AWSProperty):
    props = {
        'Name': (str, False),
        'Parameters': (dict, False),
        'SerializationLibrary': (str, False),
    }


class SkewedInfo(AWSProperty):
    props = {
        'SkewedColumnNames': ([str], False),
        'SkewedColumnValues': ([str], False),
        'SkewedColumnValueLocationMaps': (dict, False),
    }


class StorageDescriptor(AWSProperty):
    props = {
        'BucketColumns': ([str], False),
        'Columns': ([Column], False),
        'Compressed': (boolean, False),
        'InputFormat': (str, False),
        'Location': (str, False),
        'NumberOfBuckets': (positive_integer, False),
        'OutputFormat': (str, False),
        'Parameters': (dict, False),
        'SerdeInfo': (SerdeInfo, False),
        'SkewedInfo': (SkewedInfo, False),
        'SortColumns': ([Order], False),
        'StoredAsSubDirectories': (boolean, False),
    }


class PartitionInput(AWSProperty):
    props = {
        'Parameters': (dict, False),
        'StorageDescriptor': (StorageDescriptor, False),
        'Values': ([str], True),
    }


class Partition(AWSObject):
    resource_type = "AWS::Glue::Partition"

    props = {
        'CatalogId': (str, True),
        'DatabaseName': (str, True),
        'PartitionInput': (PartitionInput, True),
        'TableName': (str, True),
    }


class CloudWatchEncryption(AWSProperty):
    props = {
        'CloudWatchEncryptionMode': (str, False),
        'KmsKeyArn': (str, False),
    }


class JobBookmarksEncryption(AWSProperty):
    props = {
        'JobBookmarksEncryptionMode': (str, False),
        'KmsKeyArn': (str, False),
    }


class S3Encryptions(AWSProperty):
    props = {
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        'CloudWatchEncryption': (CloudWatchEncryption, False),
        'JobBookmarksEncryption': (JobBookmarksEncryption, False),
        'S3Encryptions': (S3Encryptions, False),
    }


class SecurityConfiguration(AWSObject):
    resource_type = "AWS::Glue::SecurityConfiguration"

    props = {
        'EncryptionConfiguration': (EncryptionConfiguration, True),
        'Name': (str, True),
    }


def table_type_validator(type):
    valid_types = [
        'EXTERNAL_TABLE',
        'VIRTUAL_VIEW',
    ]
    if type not in valid_types:
        raise ValueError('% is not a valid value for TableType' % type)
    return type


class TableInput(AWSProperty):
    props = {
        'Description': (str, False),
        'Name': (str, False),
        'Owner': (str, False),
        'Parameters': (dict, False),
        'PartitionKeys': ([Column], False),
        'Retention': (positive_integer, False),
        'StorageDescriptor': (StorageDescriptor, False),
        'TableType': (table_type_validator, False),
        'ViewExpandedText': (str, False),
        'ViewOriginalText': (str, False),
    }


class Table(AWSObject):
    resource_type = "AWS::Glue::Table"

    props = {
        'CatalogId': (str, True),
        'DatabaseName': (str, True),
        'TableInput': (TableInput, True),
    }


class Action(AWSProperty):
    props = {
        'Arguments': (dict, False),
        'CrawlerName': (str, False),
        'JobName': (str, False),
        'SecurityConfiguration': (str, False),
    }


class Condition(AWSProperty):
    props = {
        'CrawlerName': (str, False),
        'CrawlState': (str, False),
        'JobName': (str, False),
        'LogicalOperator': (str, False),
        'State': (str, False),
    }


class Predicate(AWSProperty):
    props = {
        'Conditions': ([Condition], False),
        'Logical': (str, False),
    }


def trigger_type_validator(type):
    valid_types = [
        'SCHEDULED',
        'CONDITIONAL',
        'ON_DEMAND',
    ]
    if type not in valid_types:
        raise ValueError('% is not a valid value for Type' % type)
    return type


class Trigger(AWSObject):
    resource_type = "AWS::Glue::Trigger"

    props = {
        'Actions': ([Action], True),
        'Description': (str, False),
        'Name': (str, False),
        'Predicate': (Predicate, False),
        'Schedule': (str, False),
        'StartOnCreation': (boolean, False),
        'Tags': (dict, False),
        'Type': (str, True),
        'WorkflowName': (str, False),
    }


class Workflow(AWSObject):
    resource_type = "AWS::Glue::Workflow"

    props = {
        'DefaultRunProperties': (dict, False),
        'Description': (str, False),
        'Name': (str, False),
        'Tags': (dict, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, AWSHelperFn, If, Tags
from .validators import boolean


def attribute_type_validator(x):
    valid_types = ["S", "N", "B"]
    if x not in valid_types:
        raise ValueError("AttributeType must be one of: %s" %
                         ", ".join(valid_types))
    return x


def key_type_validator(x):
    valid_types = ["HASH", "RANGE"]
    if x not in valid_types:
        raise ValueError("KeyType must be one of: %s" % ", ".join(valid_types))
    return x


def projection_type_validator(x):
    valid_types = ["KEYS_ONLY", "INCLUDE", "ALL"]
    if x not in valid_types:
        raise ValueError("ProjectionType must be one of: %s" %
                         ", ".join(valid_types))
    return x


def billing_mode_validator(x):
    valid_modes = ['PROVISIONED', 'PAY_PER_REQUEST']
    if x not in valid_modes:
        raise ValueError("Table billing mode must be one of: %s" %
                         ", ".join(valid_modes))
    return x


class AttributeDefinition(AWSProperty):
    props = {
        "AttributeName": (str, True),
        "AttributeType": (attribute_type_validator, True),
    }


class KeySchema(AWSProperty):
    props = {
        "AttributeName": (str, True),
        "KeyType": (key_type_validator, True)
    }


class Key(KeySchema):
    """ For backwards compatibility. """
    pass


class ProvisionedThroughput(AWSProperty):
    props = {
        "ReadCapacityUnits": (int, True),
        "WriteCapacityUnits": (int, True),
    }


class Projection(AWSProperty):
    props = {
        "NonKeyAttributes": ([str], False),
        "ProjectionType": (projection_type_validator, False)
    }


class SSESpecification(AWSProperty):
    props = {
        'KMSMasterKeyId': (str, False),
        'SSEEnabled': (boolean, True),
        'SSEType': (str, False),
    }


class GlobalSecondaryIndex(AWSProperty):
    props = {
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
        "ProvisionedThroughput": (ProvisionedThroughput, False)
    }


class LocalSecondaryIndex(AWSProperty):
    props = {
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
    }


class PointInTimeRecoverySpecification(AWSProperty):
    props = {
        'PointInTimeRecoveryEnabled': (boolean, False),
    }


class StreamSpecification(AWSProperty):
    props = {
        'StreamViewType': (str, True),
    }


class TimeToLiveSpecification(AWSProperty):
    props = {
        'AttributeName': (str, True),
        'Enabled': (boolean, True),
    }


class Table(AWSObject):
    resource_type = "AWS::DynamoDB::Table"

    props = {
        'AttributeDefinitions': ([AttributeDefinition], True),
        'BillingMode': (billing_mode_validator, False),
        'GlobalSecondaryIndexes': ([GlobalSecondaryIndex], False),
        'KeySchema': ([KeySchema], True),
        'LocalSecondaryIndexes': ([LocalSecondaryIndex], False),
        'PointInTimeRecoverySpecification':
            (PointInTimeRecoverySpecification, False),
        'ProvisionedThroughput': (ProvisionedThroughput, False),
        'SSESpecification': (SSESpecification, False),
        'StreamSpecification': (StreamSpecification, False),
        'TableName': (str, False),
        'Tags': (Tags, False),
        'TimeToLiveSpecification': (TimeToLiveSpecification, False),
    }

    def validate(self):
        billing_mode = self.properties.get('BillingMode', 'PROVISIONED')
        indexes = self.properties.get('GlobalSecondaryIndexes', [])
        tput_props = [self.properties]
        tput_props.extend([
            x.properties for x in indexes if not isinstance(x, AWSHelperFn)
        ])

        def check_if_all(name, props):
            validated = []
            for prop in props:
                is_helper = isinstance(prop.get(name), AWSHelperFn)
                validated.append(name in prop or is_helper)
            return all(validated)

        def check_any(name, props):
            validated = []
            for prop in props:
                is_helper = isinstance(prop.get(name), AWSHelperFn)
                validated.append(name in prop and not is_helper)
            return any(validated)

        if isinstance(billing_mode, If):
            if check_any('ProvisionedThroughput', tput_props):
                raise ValueError(
                    'Table billing mode is per-request. '
                    'ProvisionedThroughput property is mutually exclusive')
            return

        if billing_mode == 'PROVISIONED':
            if not check_if_all('ProvisionedThroughput', tput_props):
                raise ValueError(
                    'Table billing mode is provisioned. '
                    'ProvisionedThroughput required if available')
        elif billing_mode == 'PAY_PER_REQUEST':
            if check_any('ProvisionedThroughput', tput_props):
                raise ValueError(
                    'Table billing mode is per-request. '
                    'ProvisionedThroughput property is mutually exclusive')



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (integer, boolean)

VALID_STATES = ('ENABLED', 'DISABLED')
VALID_RESOURCE_TYPES = ('VOLUME')
VALID_INTERVALS = (2, 3, 4, 6, 8, 12, 24)
VALID_INTERVAL_UNITS = ('HOURS')


def validate_interval(interval):
    """Interval validation rule."""

    if interval not in VALID_INTERVALS:
        raise ValueError("Interval must be one of : %s" %
                         ", ".join(VALID_INTERVALS))
    return interval


def validate_interval_unit(interval_unit):
    """Interval unit validation rule."""

    if interval_unit not in VALID_INTERVAL_UNITS:
        raise ValueError("Interval unit must be one of : %s" %
                         ", ".join(VALID_INTERVAL_UNITS))
    return interval_unit


def validate_state(state):
    """State validation rule."""

    if state not in VALID_STATES:
        raise ValueError("State must be one of : %s" %
                         ", ".join(VALID_STATES))
    return state


class Parameters(AWSProperty):
    props = {
        'ExcludeBootVolume': (boolean, False),
    }


class CreateRule(AWSProperty):
    props = {
        'Interval': (validate_interval, True),
        'IntervalUnit': (validate_interval_unit, True),
        'Times': ([str], False),
    }


class RetainRule(AWSProperty):
    props = {
        'Count': (integer, True),
    }


class Schedule(AWSProperty):
    props = {
        'CopyTags': (boolean, False),
        'CreateRule': (CreateRule, False),
        'Name': (str, False),
        'RetainRule': (RetainRule, False),
        'TagsToAdd': ((Tags, list), False),
    }


class PolicyDetails(AWSProperty):
    props = {
        'Parameters': (Parameters, False),
        'PolicyType': (str, False),
        'ResourceTypes': ([str], False),
        'Schedules': ([Schedule], False),
        'TargetTags': ((Tags, list), False),
    }


class LifecyclePolicy(AWSObject):
    resource_type = "AWS::DLM::LifecyclePolicy"

    props = {
        'Description': (str, False),
        'ExecutionRoleArn': (str, False),
        'PolicyDetails': (PolicyDetails, False),
        'State': (validate_state, False),
    }



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSObject, AWSProperty, Tags
from .validators import (
    double
)

VALID_GROWTH_TYPE = ('LINEAR')
VALID_REPLICATION_DESTINATION = ('NONE', 'SSM_DOCUMENT')
VALID_VALIDATOR_TYPE = ('JSON_SCHEMA', 'LAMBDA')


def validate_growth_type(growth_type):
    if growth_type not in VALID_GROWTH_TYPE:
        raise ValueError("DeploymentStrategy GrowthType must be one of: %s" %
                         ', '.join(VALID_GROWTH_TYPE))
    return growth_type


def validate_replicate_to(replicate_to):
    if replicate_to not in VALID_REPLICATION_DESTINATION:
        raise ValueError("DeploymentStrategy ReplicateTo must be one of: %s" %
                         ', '.join(VALID_REPLICATION_DESTINATION))
    return replicate_to


def validate_validator_type(validator_type):
    if validator_type not in VALID_VALIDATOR_TYPE:
        raise ValueError("ConfigurationProfile Validator Type must be one of: %s" %  # NOQA
                         ', '.join(VALID_VALIDATOR_TYPE))
    return validator_type


class DeploymentStrategy(AWSObject):
    resource_type = "AWS::AppConfig::DeploymentStrategy"

    props = {
        'DeploymentDurationInMinutes': (double, True),
        'Description': (str, False),
        'FinalBakeTimeInMinutes': (double, False),
        'GrowthFactor': (double, True),
        'GrowthType': (validate_growth_type, False),
        'Name': (str, True),
        'ReplicateTo': (validate_replicate_to, True),
        'Tags': (Tags, False),
    }


class Monitors(AWSProperty):
    props = {
        'AlarmArn': (str, False),
        'AlarmRoleArn': (str, False),
    }


class Environment(AWSObject):
    resource_type = "AWS::AppConfig::Environment"

    props = {
        'ApplicationId': (str, True),
        'Description': (str, False),
        'Monitors': ([Monitors], False),
        'Name': (str, True),
        'Tags': (Tags, False),
    }


class Deployment(AWSObject):
    resource_type = "AWS::AppConfig::Deployment"

    props = {
        'ApplicationId': (str, True),
        'ConfigurationProfileId': (str, True),
        'ConfigurationVersion': (str, True),
        'DeploymentStrategyId': (str, True),
        'Description': (str, False),
        'EnvironmentId': (str, True),
        'Tags': (Tags, False),
    }


class Validators(AWSProperty):
    props = {
        'Content': (str, False),
        'Type': (validate_validator_type, False),
    }


class ConfigurationProfile(AWSObject):
    resource_type = "AWS::AppConfig::ConfigurationProfile"

    props = {
        'ApplicationId': (str, True),
        'Description': (str, False),
        'LocationUri': (str, True),
        'Name': (str, True),
        'RetrievalRoleArn': (str, False),
        'Tags': (Tags, False),
        'Validators': ([Validators], False)
    }


class Application(AWSObject):
    resource_type = "AWS::AppConfig::Application"

    props = {
        'Description': (str, False),
        'Name': (str, True),
        'Tags': (Tags, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 5.3.0


from . import AWSObject
from . import AWSProperty
from .validators import integer


class MemberFabricConfiguration(AWSProperty):
    props = {
        'AdminPassword': (str, True),
        'AdminUsername': (str, True),
    }


class MemberFrameworkConfiguration(AWSProperty):
    props = {
        'MemberFabricConfiguration': (MemberFabricConfiguration, False),
    }


class MemberConfiguration(AWSProperty):
    props = {
        'Description': (str, False),
        'MemberFrameworkConfiguration':
            (MemberFrameworkConfiguration, False),
        'Name': (str, True),
    }


class NetworkFabricConfiguration(AWSProperty):
    props = {
        'Edition': (str, True),
    }


class NetworkFrameworkConfiguration(AWSProperty):
    props = {
        'NetworkFabricConfiguration': (NetworkFabricConfiguration, False),
    }


class ApprovalThresholdPolicy(AWSProperty):
    props = {
        'ProposalDurationInHours': (integer, False),
        'ThresholdComparator': (str, False),
        'ThresholdPercentage': (integer, False),
    }


class VotingPolicy(AWSProperty):
    props = {
        'ApprovalThresholdPolicy': (ApprovalThresholdPolicy, False),
    }


class NetworkConfiguration(AWSProperty):
    props = {
        'Description': (str, False),
        'Framework': (str, True),
        'FrameworkVersion': (str, True),
        'Name': (str, True),
        'NetworkFrameworkConfiguration':
            (NetworkFrameworkConfiguration, False),
        'VotingPolicy': (VotingPolicy, True),
    }


class Member(AWSObject):
    resource_type = "AWS::ManagedBlockchain::Member"

    props = {
        'InvitationId': (str, False),
        'MemberConfiguration': (MemberConfiguration, True),
        'NetworkConfiguration': (NetworkConfiguration, False),
        'NetworkId': (str, False),
    }


class NodeConfiguration(AWSProperty):
    props = {
        'AvailabilityZone': (str, True),
        'InstanceType': (str, True),
    }


class Node(AWSObject):
    resource_type = "AWS::ManagedBlockchain::Node"

    props = {
        'MemberId': (str, True),
        'NetworkId': (str, True),
        'NodeConfiguration': (NodeConfiguration, True),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, Tags
from .validators import integer


class AssessmentTarget(AWSObject):
    resource_type = "AWS::Inspector::AssessmentTarget"

    props = {
        'AssessmentTargetName': (str, False),
        'ResourceGroupArn': (str, False),
    }


class AssessmentTemplate(AWSObject):
    resource_type = "AWS::Inspector::AssessmentTemplate"

    props = {
        'AssessmentTargetArn': (str, True),
        'AssessmentTemplateName': (str, False),
        'DurationInSeconds': (integer, True),
        'RulesPackageArns': ([str], True),
        'UserAttributesForFindings': (Tags, False),
    }


class ResourceGroup(AWSObject):
    resource_type = "AWS::Inspector::ResourceGroup"

    props = {
        'ResourceGroupTags': (Tags, True),
    }



from . import AWSObject, AWSProperty, Tags
from .compat import policytypes


class LifecyclePolicy(AWSProperty):
    props = {
        'LifecyclePolicyText': (str, False),
        'RegistryId': (str, False),
    }


class Repository(AWSObject):
    resource_type = "AWS::ECR::Repository"

    props = {
        'LifecyclePolicy': (LifecyclePolicy, False),
        'RepositoryName': (str, False),
        'RepositoryPolicyText': (policytypes, False),
        'Tags': (Tags, False),
    }




from __future__ import print_function
import argparse
import json
import pprint

try:
    getattr(__builtins__, 'basestring')
except AttributeError:
    basestring = str


class object_registry(object):
    """Keep track of objects being created as Parameters or Resources
    in order to map back to due to use in intrinsic functions like
    Ref() and GetAtt().
    """
    def __init__(self):
        self.objects = {}

    def add(self, o):
        new_name = o.replace('-', '_')
        self.objects[o] = new_name
        return new_name

    def lookup(self, o):
        if o in self.objects:
            return self.objects[o]
        else:
            return output_value(o)


objects = object_registry()

object_functions = {
    "Table":        ["ProvisionedThroughput", "PrimaryKey", "Element"],
    "LoadBalancer": ["HealthCheck", "ConnectionDrainingPolicy",
                     "AccessLoggingPolicy"],
    "Queue":        ["RedrivePolicy"],
    "Bucket":       ["WebsiteConfiguration"],
    "User":         ["LoginProfile"],
    "Topic":        ["Subscription"],
    "Instance":     ["NetworkInterfaceProperty",
                     "PrivateIpAddressSpecification"],
    "RecordSet":    ["RecordSetType"],
    "Policy":       ["PolicyType"],
}


def additional_imports(o):
    if o in object_functions:
        return ", ".join([o] + object_functions[o])
    else:
        return o


def do_header(d):
    """Output a stock header for the new Python script and also try to
    figure out the Resource imports needed by the template.
    """
    print('from troposphere import Base64, Select, FindInMap, GetAtt')
    print('from troposphere import GetAZs, Join, Output, If, And, Not')
    print('from troposphere import Or, Equals, Condition')
    print('from troposphere import Parameter, Ref, Tags, Template')
    print('from troposphere.cloudformation import Init')
    print('from troposphere.cloudfront import Distribution')
    print('from troposphere.cloudfront import DistributionConfig')
    print('from troposphere.cloudfront import Origin, DefaultCacheBehavior')
    print('from troposphere.ec2 import PortRange')

    # Loop over the resources to find imports
    if 'Resources' in d:
        seen = []
        resources = d['Resources']
        for k, v in resources.items():
            (mod, tropo_object) = generate_troposphere_object(v['Type'])
            if tropo_object not in seen:
                seen.append(tropo_object)
                print('from troposphere.{} import {}'
                      .format(mod, additional_imports(tropo_object)))
    print()
    print()
    print("t = Template()")
    print()


def do_awstemplateformatversion(d):
    """Output the template version"""
    print('t.add_version("{}")'.format(d['AWSTemplateFormatVersion']))
    print()


def do_description(d):
    """Output the template Description"""
    print('t.add_description("""\\\n{}""")'.format(d['Description']))


def do_parameters(d):
    """Output the template Parameters"""
    params = d['Parameters']
    for k, v in params.items():
        object_name = objects.add(k)
        print('{} = t.add_parameter(Parameter('.format(object_name))
        print('    "{}",'.format(k))
        for pk, pv in v.items():
            print('    {}={},'.format(pk, output_value(pv)))
        print("))")
        print()


def do_conditions(d):
    """Output the template Conditions"""
    conditions = d['Conditions']
    for k, v in conditions.items():
        print('t.add_condition("{}",'.format(k))
        print('    {}'.format(output_value(v)))
        print(")")
        print()


def do_mappings(d):
    """Output the template Mappings"""
    mappings = d['Mappings']
    for k, v in mappings.items():
        print('t.add_mapping("{}",'.format(k))
        pprint.pprint(v)
        print(")")
        print()


def map_module(mod):
    """Map module names as needed"""
    if mod == "lambda":
        return "awslambda"
    return mod


def generate_troposphere_object(typename):
    """Try to determine the troposphere object to create from the Type
    specification from the Resource being converted.
    """
    t = typename.split(':')
    if len(t) == 5:
        return (map_module(t[2].lower()), t[4])
    else:
        return ('', typename)


def output_dict(d):
    out = []
    for k, v in d.items():
        out.append("{}={}".format(k.replace('\\', '\\\\'), output_value(v)))
    return ", ".join(out)


known_functions = {
    "DistributionConfig":        1,
    "DefaultCacheBehavior":      1,
    "ProvisionedThroughput":     1,
    "NetworkInterfaces":         1,
    "WebsiteConfiguration":      1,
    "RedrivePolicy":             1,
    "Subscription":              1,
    "KeySchema":                 1,
    "HashKeyElement":            1,
    "HealthCheck":               1,
    "LoginProfile":              1,
    "ConnectionDrainingPolicy":  1,
    "AccessLoggingPolicy":       1,
    "AWS::CloudFormation::Init": 1,
    "PrivateIpAddresses":        1,
    "ContainerDefinitions":      1,
}

function_quirks = {
    "KeySchema":          "PrimaryKey",
    "HashKeyElement":     {"Element": ["AttributeName", "AttributeType"]},
    "NetworkInterfaces":  ["NetworkInterfaceProperty"],
    "Subscription":       ["Subscription"],
    "LoginProfile":       {"LoginProfile": ["Password"]},
    "AWS::CloudFormation::Init": {"Init": []},
    "PrivateIpAddresses": ["PrivateIpAddressSpecification"],
    "ContainerDefinitions": ["ContainerDefinition"],
}


def do_output_function(k, f, v):
    print('    {}={}('.format(k, f))
    for pk, pv in v.items():
        if pk in known_functions:
            do_resources_content(pk, pv, "")
        else:
            print("        {}={},".format(pk, output_value(pv)))
    print('    ),')


def do_output_quirk_list(k, f, v):
    print('    {}=['.format(k))
    for e in v:
        print('    {}('.format(f))
        for pk, pv in e.items():
            if pk in known_functions:
                do_resources_content(pk, pv)
            else:
                print("        {}={},".format(pk, output_value(pv)))
        print('    ),')
    print('    ],')


def do_output_quirk_mapping(k, v):
    m = function_quirks[k]
    for pk in m.keys():
        print('    {}={}('.format(k, pk))
        for e in m[pk]:
            print("        {},".format(output_value(v[e])))
        print('    ),')


def do_output_quirk_metadata(k, v):
    m = function_quirks[k]
    for pk in m.keys():
        print('    Metadata={}('.format(pk))
        print("        {},".format(output_value(v)))
        print('    ),')


def do_resources_content(k, v, p=""):
    if k in function_quirks:
        x = function_quirks[k]
        if(isinstance(x, dict)):
            if(p == "Metadata"):
                do_output_quirk_metadata(k, v)
            else:
                do_output_quirk_mapping(k, v)
        elif(isinstance(x, list)):
            do_output_quirk_list(k, x[0], v)
        else:
            do_output_function(k, x, v)
    else:
        do_output_function(k, k, v)


top_level_aliases = {
    "RecordSet": "RecordSetType",
    "Policy":    "PolicyType",
}


def do_resources(d):
    """Output the template Resources"""

    resources = d['Resources']
    for k, v in resources.items():
        object_name = objects.add(k)
        (_, tropo_object) = generate_troposphere_object(v['Type'])
        if tropo_object in top_level_aliases:
            tropo_object = top_level_aliases[tropo_object]
        print('{} = t.add_resource({}('.format(object_name, tropo_object))
        print('    "{}",'.format(k))
        for p in filter(lambda x: x in v, ['Metadata', 'Properties']):
            for pk, pv in v[p].items():
                if pk == "Tags":
                    print('    Tags=Tags(')
                    for d in pv:
                        print("        {}={},".format(
                            d['Key'], output_value(d['Value'])))
                    print('    ),')
                elif pk == 'PortRange':
                    print("    {}={}({}),".format(pk, pk, output_dict(pv)))
                elif pk in known_functions:
                    do_resources_content(pk, pv, p)
                elif isinstance(pv, basestring):
                    print('    {}="{}",'.format(pk, pv))
                else:
                    print('    {}={},'.format(pk, output_value(pv)))
        if "DependsOn" in v:
            print('    {}={},'
                  .format("DependsOn", output_value(v['DependsOn'])))
        if "Condition" in v:
            print('    {}={},'
                  .format("Condition", output_value(v['Condition'])))
        print("))")
        print()


def handle_no_objects(name, values):
    """Handle intrinsic functions which do not have a named resource"""
    return name + "(" + ", ".join(map(output_value, values)) + ")"


def handle_one_object(name, values):
    """Handle intrinsic functions which have a single named resource"""
    ret = name + "("
    for i, param in enumerate(values):
        if i > 0:
            ret += ", "
        # First parameter might be an object name or pseudo parameter
        if i == 0:
            ret += objects.lookup(param)
        else:
            ret += output_value(param)
    return ret + ")"


function_map = {
    'Fn::Base64': ("Base64", handle_no_objects),
    'Fn::And': ("And", handle_no_objects),
    'Fn::Or': ("Or", handle_no_objects),
    'Fn::Not': ("Not", handle_no_objects),
    'Fn::If': ("If", handle_one_object),
    'Fn::Equals': ("Equals", handle_no_objects),
    'Fn::FindInMap': ("FindInMap", handle_no_objects),
    'Fn::GetAtt': ("GetAtt", handle_one_object),
    'Fn::GetAZs': ("GetAZs", handle_no_objects),
    'Fn::Join': ("Join", handle_no_objects),
    'Fn::Select': ("Select", handle_one_object),
    'Ref': ("Ref", handle_one_object),
    'Condition': ("Condition", handle_one_object),
}


def output_value(v):
    """Output a value which may be a string or a set of function calls."""

    if isinstance(v, basestring):
        return '"{}"'.format(v.replace('\\', '\\\\')
                             .replace('\n', '\\n')
                             .replace("\"", "\\\""))
    elif isinstance(v, bool):
        return '{}'.format(str(v))
    elif isinstance(v, int):
        return '{}'.format(v)
    elif isinstance(v, float):
        return '{}'.format(v)
    elif isinstance(v, list):
        return "[" + ", ".join(map(output_value, v)) + "]"

    out = []
    # Should only be one of these...
    for fk, fv in v.items():
        if fk in function_map:
            (shortname, handler) = function_map[fk]
            if not isinstance(fv, list):
                fv = [fv]
            return handler(shortname, fv)
        else:
            out.append('"' + fk + '": ' + output_value(fv))
    return "{ " + ", ".join(out) + " }"


def do_outputs(d):
    """Output the template Outputs"""

    outputs = d['Outputs']
    for k, v in outputs.items():
        print('{} = t.add_output(Output('.format(k))
        print('    "{}",'.format(k))
        for pk, pv in v.items():
            if isinstance(pv, basestring):
                print('    {}="{}",'.format(pk, pv))
            else:
                print('    {}={},'.format(pk, output_value(pv)))
        print("))")
        print()


def do_trailer(d):
    """Output a trailer section for the new Python script."""
    print('print(t.to_json())')


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("filename", help="template to convert")
    args = parser.parse_args()

    d = json.load(open(args.filename))

    do_header(d)

    sections = [
        'AWSTemplateFormatVersion',
        'Description',
        'Parameters',
        'Conditions',
        'Mappings',
        'Resources',
        'Outputs',
    ]

    for s in sections:
        if s in d.keys():
            globals()["do_" + s.lower()](d)

    do_trailer(d)



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject
from .validators import boolean, integer
from troposphere import Tags


class DBClusterParameterGroup(AWSObject):
    resource_type = "AWS::Neptune::DBClusterParameterGroup"

    props = {
        'Description': (str, True),
        'Family': (str, True),
        'Name': (str, False),
        'Parameters': (dict, True),
        'Tags': (Tags, False),
    }


class DBCluster(AWSObject):
    resource_type = "AWS::Neptune::DBCluster"

    props = {
        'AvailabilityZones': ([str], False),
        'BackupRetentionPeriod': (integer, False),
        'DBClusterIdentifier': (str, False),
        'DBClusterParameterGroupName': (str, False),
        'DBSubnetGroupName': (str, False),
        'DeletionProtection': (boolean, False),
        'EnableCloudwatchLogsExports': ([str], False),
        'EngineVersion': (str, False),
        'IamAuthEnabled': (boolean, False),
        'KmsKeyId': (str, False),
        'Port': (integer, False),
        'PreferredBackupWindow': (str, False),
        'PreferredMaintenanceWindow': (str, False),
        'SnapshotIdentifier': (str, False),
        'StorageEncrypted': (boolean, False),
        'Tags': (Tags, False),
        'VpcSecurityGroupIds': ([str], False),
    }


class DBInstance(AWSObject):
    resource_type = "AWS::Neptune::DBInstance"

    props = {
        'AllowMajorVersionUpgrade': (boolean, False),
        'AutoMinorVersionUpgrade': (boolean, False),
        'AvailabilityZone': (str, False),
        'DBClusterIdentifier': (str, False),
        'DBInstanceClass': (str, True),
        'DBInstanceIdentifier': (str, False),
        'DBParameterGroupName': (str, False),
        'DBSnapshotIdentifier': (str, False),
        'DBSubnetGroupName': (str, False),
        'PreferredMaintenanceWindow': (str, False),
        'Tags': (Tags, False),
    }


class DBParameterGroup(AWSObject):
    resource_type = "AWS::Neptune::DBParameterGroup"

    props = {
        'Description': (str, True),
        'Family': (str, True),
        'Name': (str, False),
        'Parameters': (dict, True),
        'Tags': (Tags, False),
    }


class DBSubnetGroup(AWSObject):
    resource_type = "AWS::Neptune::DBSubnetGroup"

    props = {
        'DBSubnetGroupDescription': (str, True),
        'DBSubnetGroupName': (str, False),
        'SubnetIds': ([str], True),
        'Tags': (Tags, False),
    }



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, BaseAWSObject, Tags
from . import encode_to_dict
from .validators import boolean, check_required, encoding, integer


class Stack(AWSObject):
    resource_type = "AWS::CloudFormation::Stack"

    props = {
        'NotificationARNs': ([str], False),
        'Parameters': (dict, False),
        'Tags': ((Tags, list), False),
        'TemplateURL': (str, True),
        'TimeoutInMinutes': (integer, False),
    }


class AWSCustomObject(BaseAWSObject):
    dictname = 'Properties'


class CustomResource(AWSCustomObject):
    resource_type = "AWS::CloudFormation::CustomResource"

    props = {
        'ServiceToken': (str, True)
    }


class WaitCondition(AWSObject):
    resource_type = "AWS::CloudFormation::WaitCondition"

    props = {
        'Count': (integer, False),
        'Handle': (str, False),
        'Timeout': (integer, False),
    }

    def validate(self):
        if 'CreationPolicy' in self.resource:
            for k in list(self.props.keys()):
                if k in self.properties:
                    raise ValueError(
                        "Property %s cannot be specified with CreationPolicy" %
                        k
                    )
        else:
            required = ['Handle', 'Timeout']
            check_required(self.__class__.__name__, self.properties, required)


class WaitConditionHandle(AWSObject):
    resource_type = "AWS::CloudFormation::WaitConditionHandle"

    props = {}


class Metadata(AWSHelperFn):
    def __init__(self, *args):
        self.data = args

    def to_dict(self):
        t = []
        for i in self.data:
            t += list(encode_to_dict(i).items())
        return dict(t)


class InitFileContext(AWSHelperFn):
    def __init__(self, data):
        self.data = data


class InitFile(AWSProperty):
    props = {
        'content': (str, False),
        'mode': (str, False),
        'owner': (str, False),
        'encoding': (encoding, False),
        'group': (str, False),
        'source': (str, False),
        'authentication': (str, False),
        'context': (InitFileContext, False)
    }


class InitFiles(AWSHelperFn):
    def __init__(self, data):
        self.validate(data)
        self.data = data

    def validate(self, data):
        for k in data:
            if not isinstance(data[k], InitFile):
                raise ValueError("File '" + k + "' must be of type InitFile")


class InitService(AWSProperty):
    props = {
        'ensureRunning': (boolean, False),
        'enabled': (boolean, False),
        'files': (list, False),
        'packages': (dict, False),
        'sources': (list, False),
        'commands': (list, False)
    }


class InitServices(AWSHelperFn):
    def __init__(self, data):
        self.validate(data)
        self.data = data

    def validate(self, data):
        for k in data:
            if not isinstance(data[k], InitService):
                raise ValueError(
                    "Service '" + k + "' must be of type InitService"
                )


class InitConfigSets(AWSHelperFn):
    def __init__(self, **kwargs):
        self.validate(dict(kwargs))
        self.data = kwargs

    def validate(self, config_sets):
        for k, v in config_sets.items():
            if not isinstance(v, list):
                raise ValueError('configSets values must be of type list')


class InitConfig(AWSProperty):
    props = {
        'groups': (dict, False),
        'users': (dict, False),
        'sources': (dict, False),
        'packages': (dict, False),
        'files': (dict, False),
        'commands': (dict, False),
        'services': (dict, False)
    }


def validate_authentication_type(auth_type):
    valid_types = ['S3', 'basic']
    if auth_type not in valid_types:
        raise ValueError('Type needs to be one of %r' % valid_types)
    return auth_type


class AuthenticationBlock(AWSProperty):
    props = {
        "accessKeyId": (str, False),
        "buckets": ([str], False),
        "password": (str, False),
        "secretKey": (str, False),
        "type": (validate_authentication_type, False),
        "uris": ([str], False),
        "username": (str, False),
        "roleName": (str, False)
    }


class Authentication(AWSHelperFn):
    def __init__(self, data):
        self.validate(data)
        self.data = {"AWS::CloudFormation::Authentication": data}

    def validate(self, data):
        for k, v in data.items():
            if not isinstance(v, AuthenticationBlock):
                raise ValueError(
                    'authentication block must be of type'
                    ' cloudformation.AuthenticationBlock'
                )


class Init(AWSHelperFn):
    def __init__(self, data, **kwargs):
        self.validate(data, dict(kwargs))

        if isinstance(data, InitConfigSets):
            self.data = {
                'AWS::CloudFormation::Init': dict({'configSets': data},
                                                  **kwargs)
            }
        else:
            self.data = {'AWS::CloudFormation::Init': data}

    def validate(self, data, config_sets):
        if isinstance(data, InitConfigSets):
            for k, v in sorted(config_sets.items()):
                if not isinstance(v, InitConfig):
                    raise ValueError(
                        'init configs must of type ',
                        'cloudformation.InitConfigSet'
                    )
        else:
            if 'config' not in data:
                raise ValueError('config property is required')
            if not isinstance(data['config'], InitConfig):
                raise ValueError(
                    'config property must be of type cloudformation.InitConfig'
                )


class Macro(AWSCustomObject):
    resource_type = "AWS::CloudFormation::Macro"

    props = {
        'Description': (str, False),
        'FunctionName': (str, True),
        'LogGroupName': (str, False),
        'LogRoleARN': (str, False),
        'Name': (str, True),
    }



# Copyright (c) 2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, positive_integer


VALID_RECOVERYOPTION_NAME = (
    'admin_only', 'verified_email', 'verified_phone_number')


def validate_recoveryoption_name(recoveryoption_name):
    """Validate Name for RecoveryOption"""

    if recoveryoption_name not in VALID_RECOVERYOPTION_NAME:
        raise ValueError("RecoveryOption Name must be one of: %s" %
                         ", ".join(VALID_RECOVERYOPTION_NAME))
    return recoveryoption_name


class CognitoIdentityProvider(AWSProperty):
    props = {
        'ClientId': (str, False),
        'ProviderName': (str, False),
        'ServerSideTokenCheck': (bool, False),
    }


class CognitoStreams(AWSProperty):
    props = {
        'RoleArn': (str, False),
        'StreamingStatus': (str, False),
        'StreamName': (str, False),
    }


class PushSync(AWSProperty):
    props = {
        'ApplicationArns': ([str], False),
        'RoleArn': (str, False),
    }


class IdentityPool(AWSObject):
    resource_type = "AWS::Cognito::IdentityPool"

    props = {
        'AllowUnauthenticatedIdentities': (bool, True),
        'CognitoEvents': (dict, False),
        'CognitoIdentityProviders': ([CognitoIdentityProvider], False),
        'CognitoStreams': (CognitoStreams, False),
        'DeveloperProviderName': (str, False),
        'IdentityPoolName': (str, False),
        'OpenIdConnectProviderARNs': ([str], False),
        'PushSync': (PushSync, False),
        'SamlProviderARNs': ([str], False),
        'SupportedLoginProviders': (dict, False),
    }


class MappingRule(AWSProperty):
    props = {
        'Claim': (str, True),
        'MatchType': (str, True),
        'RoleARN': (str, True),
        'Value': (str, True),
    }


class RulesConfiguration(AWSProperty):
    props = {
        'Rules': ([MappingRule], True),
    }


class RoleMapping(AWSProperty):
    props = {
        'AmbiguousRoleResolution': (str, False),
        'RulesConfiguration': (RulesConfiguration, False),
        'Type': (str, True),
    }


class IdentityPoolRoleAttachment(AWSObject):
    resource_type = "AWS::Cognito::IdentityPoolRoleAttachment"

    props = {
        'IdentityPoolId': (str, True),
        'RoleMappings': (dict, False),
        'Roles': (dict, False),
    }


class InviteMessageTemplate(AWSProperty):
    props = {
        'EmailMessage': (str, False),
        'EmailSubject': (str, False),
        'SMSMessage': (str, False),
    }


class AdminCreateUserConfig(AWSProperty):
    props = {
        'AllowAdminCreateUserOnly': (boolean, False),
        'InviteMessageTemplate': (InviteMessageTemplate, False),
        'UnusedAccountValidityDays': (positive_integer, False),
    }


class DeviceConfiguration(AWSProperty):
    props = {
        'ChallengeRequiredOnNewDevice': (boolean, False),
        'DeviceOnlyRememberedOnUserPrompt': (boolean, False),
    }


class EmailConfiguration(AWSProperty):
    props = {
        'ConfigurationSet': (str, False),
        'EmailSendingAccount': (str, False),
        'From': (str, False),
        'ReplyToEmailAddress': (str, False),
        'SourceArn': (str, False),
    }


class LambdaConfig(AWSProperty):
    props = {
        'CreateAuthChallenge': (str, False),
        'CustomMessage': (str, False),
        'DefineAuthChallenge': (str, False),
        'PostAuthentication': (str, False),
        'PostConfirmation': (str, False),
        'PreAuthentication': (str, False),
        'PreSignUp': (str, False),
        'PreTokenGeneration': (str, False),
        'UserMigration': (str, False),
        'VerifyAuthChallengeResponse': (str, False),
    }


class PasswordPolicy(AWSProperty):
    props = {
        'MinimumLength': (positive_integer, False),
        'RequireLowercase': (boolean, False),
        'RequireNumbers': (boolean, False),
        'RequireSymbols': (boolean, False),
        'RequireUppercase': (boolean, False),
        'TemporaryPasswordValidityDays': (positive_integer, False),
    }


class Policies(AWSProperty):
    props = {
        'PasswordPolicy': (PasswordPolicy, False),
    }


class NumberAttributeConstraints(AWSProperty):
    props = {
        'MaxValue': (str, False),
        'MinValue': (str, False),
    }


class StringAttributeConstraints(AWSProperty):
    props = {
        'MaxLength': (str, False),
        'MinLength': (str, False),
    }


class SchemaAttribute(AWSProperty):
    props = {
        'AttributeDataType': (str, False),
        'DeveloperOnlyAttribute': (boolean, False),
        'Mutable': (boolean, False),
        'Name': (str, False),
        'NumberAttributeConstraints': (NumberAttributeConstraints, False),
        'StringAttributeConstraints': (StringAttributeConstraints, False),
        'Required': (boolean, False),
    }


class SmsConfiguration(AWSProperty):
    props = {
        'ExternalId': (str, False),
        'SnsCallerArn': (str, True),
    }


class UserPoolAddOns(AWSProperty):
    props = {
        'AdvancedSecurityMode': (str, False),
    }


class VerificationMessageTemplate(AWSProperty):
    props = {
        'DefaultEmailOption': (str, False),
        'EmailMessage': (str, False),
        'EmailMessageByLink': (str, False),
        'EmailSubject': (str, False),
        'EmailSubjectByLink': (str, False),
        'SmsMessage': (str, False),
    }


class RecoveryOption(AWSProperty):
    props = {
        'Name': (validate_recoveryoption_name, False),
        'Priority': (positive_integer, False)
    }


class AccountRecoverySetting(AWSProperty):
    props = {
        'RecoveryMechanisms': ([RecoveryOption], False)
    }


class UsernameConfiguration(AWSProperty):
    props = {
        'CaseSensitive': (boolean, False),
    }


class UserPool(AWSObject):
    resource_type = "AWS::Cognito::UserPool"

    props = {
        'AccountRecoverySetting': (AccountRecoverySetting, False),
        'AdminCreateUserConfig': (AdminCreateUserConfig, False),
        'AliasAttributes': ([str], False),
        'AutoVerifiedAttributes': ([str], False),
        'DeviceConfiguration': (DeviceConfiguration, False),
        'EmailConfiguration': (EmailConfiguration, False),
        'EmailVerificationMessage': (str, False),
        'EmailVerificationSubject': (str, False),
        'EnabledMfas': ([str], False),
        'LambdaConfig': (LambdaConfig, False),
        'MfaConfiguration': (str, False),
        'Policies': (Policies, False),
        'Schema': ([SchemaAttribute], False),
        'SmsAuthenticationMessage': (str, False),
        'SmsConfiguration': (SmsConfiguration, False),
        'SmsVerificationMessage': (str, False),
        'UserPoolAddOns': (UserPoolAddOns, False),
        'UserPoolName': (str, False),
        'UserPoolTags': (dict, False),
        'UsernameAttributes': ([str], False),
        'UsernameConfiguration': (UsernameConfiguration, False),
        'VerificationMessageTemplate': (VerificationMessageTemplate, False),
    }


class AnalyticsConfiguration(AWSProperty):
    props = {
        'ApplicationId': (str, False),
        'ExternalId': (str, False),
        'RoleArn': (str, False),
        'UserDataShared': (boolean, False),
    }


class UserPoolClient(AWSObject):
    resource_type = "AWS::Cognito::UserPoolClient"

    props = {
        'AllowedOAuthFlows': ([str], False),
        'AllowedOAuthFlowsUserPoolClient': (boolean, False),
        'AllowedOAuthScopes': ([str], False),
        'AnalyticsConfiguration': (AnalyticsConfiguration, False),
        'CallbackURLs': ([str], False),
        'ClientName': (str, False),
        'DefaultRedirectURI': (str, False),
        'ExplicitAuthFlows': ([str], False),
        'GenerateSecret': (boolean, False),
        'LogoutURLs': ([str], False),
        'PreventUserExistenceErrors': (str, False),
        'ReadAttributes': ([str], False),
        'RefreshTokenValidity': (positive_integer, False),
        'SupportedIdentityProviders': ([str], False),
        'UserPoolId': (str, True),
        'WriteAttributes': ([str], False),
    }


class CustomDomainConfigType(AWSProperty):
    props = {
        'CertificateArn': (str, False),
    }


class UserPoolDomain(AWSObject):
    resource_type = "AWS::Cognito::UserPoolDomain"

    props = {
        'CustomDomainConfig': (CustomDomainConfigType, False),
        'Domain': (str, True),
        'UserPoolId': (str, True),
    }


class UserPoolGroup(AWSObject):
    resource_type = "AWS::Cognito::UserPoolGroup"

    props = {
        'Description': (str, False),
        'GroupName': (str, True),
        'Precedence': (positive_integer, False),
        'RoleArn': (str, False),
        'UserPoolId': (str, True),
    }


class UserPoolIdentityProvider(AWSObject):
    resource_type = "AWS::Cognito::UserPoolIdentityProvider"

    props = {
        'AttributeMapping': (dict, False),
        'IdpIdentifiers': ([str], False),
        'ProviderDetails': (dict, False),
        'ProviderName': (str, True),
        'ProviderType': (str, True),
        'UserPoolId': (str, True),
    }


class ResourceServerScopeType(AWSProperty):
    props = {
        "ScopeDescription": (str, True),
        "ScopeName": (str, True)
    }


class UserPoolResourceServer(AWSObject):
    resource_type = "AWS::Cognito::UserPoolResourceServer"

    props = {
        "Identifier": (str, True),
        "Name": (str, True),
        "Scopes": ([ResourceServerScopeType], False),
        "UserPoolId": (str, True)
    }


class AccountTakeoverActionType(AWSProperty):
    props = {
        'EventAction': (str, True),
        'Notify': (boolean, True),
    }


class AccountTakeoverActionsType(AWSProperty):
    props = {
        'HighAction': (AccountTakeoverActionType, False),
        'LowAction': (AccountTakeoverActionType, False),
        'MediumAction': (AccountTakeoverActionType, False),
    }


class NotifyEmailType(AWSProperty):
    props = {
        'HtmlBody': (str, False),
        'Subject': (str, True),
        'TextBody': (str, False),
    }


class NotifyConfigurationType(AWSProperty):
    props = {
        'BlockEmail': (NotifyEmailType, False),
        'From': (str, False),
        'MfaEmail': (NotifyEmailType, False),
        'NoActionEmail': (NotifyEmailType, False),
        'ReplyTo': (str, False),
        'SourceArn': (str, True),
    }


class AccountTakeoverRiskConfigurationType(AWSProperty):
    props = {
        'Actions': (AccountTakeoverActionsType, True),
        'NotifyConfiguration': (NotifyConfigurationType, False),
    }


class CompromisedCredentialsActionsType(AWSProperty):
    props = {
        'EventAction': (str, True),
    }


class CompromisedCredentialsRiskConfigurationType(AWSProperty):
    props = {
        'Actions': (CompromisedCredentialsActionsType, True),
        'EventFilter': ([str], False),
    }


class RiskExceptionConfigurationType(AWSProperty):
    props = {
        'BlockedIPRangeList': ([str], False),
        'SkippedIPRangeList': ([str], False),
    }


class UserPoolRiskConfigurationAttachment(AWSObject):
    resource_type = "AWS::Cognito::UserPoolRiskConfigurationAttachment"

    props = {
        'AccountTakeoverRiskConfiguration':
            (AccountTakeoverRiskConfigurationType, False),
        'ClientId': (str, True),
        'CompromisedCredentialsRiskConfiguration':
            (CompromisedCredentialsRiskConfigurationType, False),
        'RiskExceptionConfiguration':
            (RiskExceptionConfigurationType, False),
        'UserPoolId': (str, True),
    }


class UserPoolUICustomizationAttachment(AWSObject):
    resource_type = "AWS::Cognito::UserPoolUICustomizationAttachment"

    props = {
        'CSS': (str, False),
        'ClientId': (str, True),
        'UserPoolId': (str, True),
    }


class AttributeType(AWSProperty):
    props = {
        'Name': (str, True),
        'Value': (str, False),
    }


class UserPoolUser(AWSObject):
    resource_type = "AWS::Cognito::UserPoolUser"

    props = {
        'ClientMetadata': (dict, False),
        'DesiredDeliveryMediums': ([str], False),
        'ForceAliasCreation': (boolean, False),
        'UserAttributes': ([AttributeType], False),
        'MessageAction': (str, False),
        'Username': (str, False),
        'UserPoolId': (str, True),
        'ValidationData': ([AttributeType], False),
    }


class UserPoolUserToGroupAttachment(AWSObject):
    resource_type = "AWS::Cognito::UserPoolUserToGroupAttachment"

    props = {
        'GroupName': (str, True),
        'Username': (str, True),
        'UserPoolId': (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


VALID_TRANSFORMATION_TYPES = (
    'CMD_LINE', 'COMPRESS_WHITE_SPACE', 'HTML_ENTITY_DECODE',
    'LOWERCASE', 'NONE', 'URL_DECODE')
VALID_COMPARISON_OPERATORS = ('EQ', 'GE', 'GT', 'LE', 'LT', 'NE')
VALID_IP_VERSION = ('IPV4', 'IPV6')
VALID_POSITIONAL_CONSTRAINTS = ('CONTAINS', 'CONTAINS_WORD', 'ENDS_WITH',
                                'EXACTLY', 'STARTS_WITH')


def validate_transformation_type(transformation_type):
    """Validate Transformation Type for WebACL TextTransformation"""

    if transformation_type not in VALID_TRANSFORMATION_TYPES:
        raise ValueError("WebACL TextTransformation must be one of: %s" %
                         ", ".join(VALID_TRANSFORMATION_TYPES))
    return transformation_type


def validate_comparison_operator(comparison_operator):
    """Validate Comparison Operator for WebACL SizeConstraintStatement"""

    if comparison_operator not in VALID_COMPARISON_OPERATORS:
        raise ValueError("WebACL SizeConstraintStatement must be one of: %s" %
                         ", ".join(VALID_COMPARISON_OPERATORS))
    return comparison_operator


def validate_ipaddress_version(ipaddress_version):
    """Validate IPAddress version for IPSet"""

    if ipaddress_version not in VALID_IP_VERSION:
        raise ValueError("IPSet IPAddressVersion must be one of: %s" %
                         ", ".join(VALID_IP_VERSION))
    return ipaddress_version


def validate_positional_constraint(positional_constraint):
    """Validate positional constraint for ByteMatchStatement"""

    if positional_constraint not in VALID_POSITIONAL_CONSTRAINTS:
        raise ValueError("ByteMatchStatement PositionalConstraint must be one of: %s" %  # NOQA
                         ", ".join(VALID_POSITIONAL_CONSTRAINTS))
    return positional_constraint


class ExcludedRule(AWSProperty):
    props = {
        'Name': (str, False)
    }


class ExcludedRules(AWSProperty):
    props = {
        'ExcludedRules': ([ExcludedRule], False)
    }


class RuleGroupReferenceStatement(AWSProperty):
    props = {
        'Arn': (str, False),
        'ExcludedRules': (ExcludedRules, False)
    }


class TextTransformation(AWSProperty):
    props = {
        'Priority': (integer, False),
        'Type': (validate_transformation_type, False),
    }


class TextTransformations(AWSProperty):
    props = {
        'TextTransformations': ([TextTransformation], False)
    }


class SingleHeader(AWSProperty):
    props = {
        'Name': (str, False)
    }


class SingleQueryArgument(AWSProperty):
    props = {
        'Name': (str, False)
    }


class Body(AWSObject):
    props = {

    }


class Method(AWSObject):
    props = {

    }


class AllQueryArguments(AWSObject):
    props = {

    }


class QueryString(AWSObject):
    props = {

    }


class UriPath(AWSObject):
    props = {

    }


class FieldToMatch(AWSProperty):
    props = {
        'AllQueryArguments': (AllQueryArguments, False),
        'Body': (Body, False),
        'Method': (Method, False),
        'QueryString': (QueryString, False),
        'SingleHeader': (SingleHeader, False),
        'SingleQueryArgument': (SingleQueryArgument, False),
        'UriPath': (UriPath, False)
    }


class RegexPatternSetReferenceStatement(AWSProperty):
    props = {
        'Arn': (str, False),
        'FieldToMatch': (FieldToMatch, False),
        'TextTransformations': (TextTransformations, False)
    }


class XssMatchStatement(AWSProperty):
    props = {
        'FieldToMatch': (FieldToMatch, False),
        'TextTransformations': (TextTransformations, False)
    }


class SqliMatchStatement(AWSProperty):
    props = {
        'FieldToMatch': (FieldToMatch, False),
        'TextTransformations': (TextTransformations, False)
    }


class SizeConstraintStatement(AWSProperty):
    props = {
        'ComparisonOperator': (validate_comparison_operator, False),
        'FieldToMatch': (FieldToMatch, False),
        'Size': (integer, False),
        'TextTransformations': (TextTransformations, False)
    }


class ByteMatchStatement(AWSProperty):
    props = {
        'FieldToMatch': (FieldToMatch, False),
        'PositionalConstraint': (validate_positional_constraint, False),
        'SearchString': (str, False),
        'SearchStringBase64': (str, False),
        'TextTransformations': (TextTransformations, False)
    }


class CountryCodes(AWSProperty):
    props = {
        'CountryCodes': ([str], False)
    }


class GeoMatchStatement(AWSProperty):
    props = {
        'CountryCodes': (CountryCodes, False)
    }


class IPSetReferenceStatement(AWSProperty):
    props = {
        'Arn': (str, False)
    }


class ManagedRuleGroupStatement(AWSProperty):
    props = {
        'ExcludedRules': (ExcludedRules, False),
        'Name': (str, False),
        'VendorName': (str, False),
    }


class StatementThree(AWSProperty):
    props = {
        'ByteMatchStatement': (ByteMatchStatement, False),
        'GeoMatchStatement': (GeoMatchStatement, False),
        'IPSetReferenceStatement': (IPSetReferenceStatement, False),
        'ManagedRuleGroupStatement': (ManagedRuleGroupStatement, False),
        'RegexPatternSetReferenceStatement': (
            RegexPatternSetReferenceStatement,
            False),
        'RuleGroupReferenceStatement': (RuleGroupReferenceStatement, False),
        'SizeConstraintStatement': (SizeConstraintStatement, False),
        'SqliMatchStatement': (SqliMatchStatement, False),
        'XssMatchStatement': (XssMatchStatement, False),
    }


class StatementThrees(AWSProperty):
    props = {
        'StatementThrees': ([StatementThree], False)
    }


class AndStatementTwo(AWSProperty):
    props = {
        'Statements': (StatementThrees, False)
    }


class NotStatementTwo(AWSProperty):
    props = {
        'Statement': (StatementThree, False)
    }


class OrStatementTwo(AWSProperty):
    props = {
        'Statements': (StatementThrees, False)
    }


class RateBasedStatementTwo(AWSProperty):
    props = {
        'AggregateKeyType': (str, False),
        'Limit': (integer, False),
        'ScopeDownStatement': StatementThree
    }


class StatementTwo(AWSProperty):
    props = {
        'AndStatement': (AndStatementTwo, False),
        'ByteMatchStatement': (ByteMatchStatement, False),
        'GeoMatchStatement': (GeoMatchStatement, False),
        'IPSetReferenceStatement': (IPSetReferenceStatement, False),
        'ManagedRuleGroupStatement': (ManagedRuleGroupStatement, False),
        'NotStatement': (NotStatementTwo, False),
        'OrStatement': (OrStatementTwo, False),
        'RateBasedStatement': (RateBasedStatementTwo, False),
        'RegexPatternSetReferenceStatement': (
            RegexPatternSetReferenceStatement,
            False),
        'RuleGroupReferenceStatement': (RuleGroupReferenceStatement, False),
        'SizeConstraintStatement': (SizeConstraintStatement, False),
        'SqliMatchStatement': (SqliMatchStatement, False),
        'XssMatchStatement': (XssMatchStatement, False),
    }


class StatementTwos(AWSProperty):
    props = {
        'StatementTwos': ([StatementTwo], False)
    }


class AndStatementOne(AWSProperty):
    props = {
        'Statements': (StatementTwos, False)
    }


class NotStatementOne(AWSProperty):
    props = {
        'Statement': (StatementTwo, False)
    }


class OrStatementOne(AWSProperty):
    props = {
        'Statements': (StatementTwos, False)
    }


class RateBasedStatementOne(AWSProperty):
    props = {
        'AggregateKeyType': (str, False),
        'Limit': (integer, False),
        'ScopeDownStatement': (StatementTwo, False)
    }


class StatementOne(AWSProperty):
    props = {
        'AndStatement': (AndStatementOne, False),
        'ByteMatchStatement': (ByteMatchStatement, False),
        'GeoMatchStatement': (GeoMatchStatement, False),
        'IPSetReferenceStatement': (IPSetReferenceStatement, False),
        'ManagedRuleGroupStatement': (ManagedRuleGroupStatement, False),
        'NotStatement': (NotStatementOne, False),
        'OrStatement': (OrStatementOne, False),
        'RateBasedStatement': (RateBasedStatementOne, False),
        'RegexPatternSetReferenceStatement': (
            RegexPatternSetReferenceStatement,
            False),
        'RuleGroupReferenceStatement': (RuleGroupReferenceStatement, False),
        'SizeConstraintStatement': (SizeConstraintStatement, False),
        'SqliMatchStatement': (SqliMatchStatement, False),
        'XssMatchStatement': (XssMatchStatement, False),
    }


class VisibilityConfig(AWSProperty):
    props = {
        'CloudWatchMetricsEnabled': (boolean, False),
        'MetricName': (str, False),
        'SampledRequestsEnabled': (boolean, False)
    }


class AllowAction(AWSProperty):
    props = {

    }


class BlockAction(AWSProperty):
    props = {

    }


class CountAction(AWSProperty):
    props = {

    }


class NoneAction(AWSProperty):
    props = {

    }


class RuleAction(AWSProperty):
    props = {
        'Allow': (AllowAction, False),
        'Block': (BlockAction, False),
        'Count': (CountAction, False),
    }


class OverrideAction(AWSProperty):
    props = {
        'Count': (CountAction, False),
        'None': (NoneAction, False),
    }


class Rule(AWSProperty):
    props = {
        'Action': (RuleAction, False),
        'Name': (str, False),
        'OverrideAction': (OverrideAction, False),
        'Priority': (integer, False),
        'Statement': (StatementOne, False),
        'VisibilityConfig': (VisibilityConfig, False)
    }


class Rules(AWSProperty):
    props = {
        'Rules': ([Rule], False),
    }


class DefaultAction(AWSProperty):
    props = {
        'Allow': (AllowAction, False),
        'Block': (BlockAction, False),
    }


class WebACL(AWSObject):
    resource_type = "AWS::WAFv2::WebACL"

    props = {
        'DefaultAction': (DefaultAction, False),
        'Description': (str, False),
        'Name': (str, True),
        'Rules': (Rules, False),
        'Scope': (str, True),
        'Tags': (Tags, False),
        'VisibilityConfig': (VisibilityConfig, False)
    }


class IPSet(AWSObject):
    resource_type = "AWS::WAFv2::IPSet"

    props = {
        'Addresses': ([str], False),
        'Description': (str, False),
        'IPAddressVersion': (validate_ipaddress_version, False),
        'Name': (str, True),
        'Scope': (str, True),
        'Tags': (Tags, False),
    }


class Regex(AWSProperty):
    props = {
        'RegexString': (str, False)
    }


class RegularExpressionList(AWSProperty):
    props = {
        'RegularExpressionList': ([Regex], False)
    }


class RegexPatternSet(AWSObject):
    resource_type = "AWS::WAFv2::RegexPatternSet"

    props = {
        'Description': (str, False),
        'Name': (str, True),
        'RegularExpressionList': (RegularExpressionList, False),
        'Scope': (str, True),
        'Tags': (Tags, False),
    }


class RuleGroup(AWSObject):
    resource_type = "AWS::WAFv2::RuleGroup"

    props = {
        'Capacity': (integer, False),
        'Description': (str, False),
        'Name': (str, True),
        'Rules': (Rules, False),
        'Scope': (str, False),
        'Tags': (Tags, False),
        'VisibilityConfig': (VisibilityConfig, False)
    }


class WebACLAssociation(AWSObject):
    resource_type = "AWS::WAFv2::WebACLAssociation"

    props = {
        'ResourceArn': (str, True),
        'WebACLArn': (str, True),
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, integer


ONE_HOUR = "One_Hour"
THREE_HOURS = "Three_Hours"
SIX_HOURS = "Six_Hours"
TWELVE_HOURS = "Twelve_Hours"
TWENTYFOUR_HOURS = "TwentyFour_Hours"


class Scope(AWSProperty):
    props = {
        'ComplianceResourceId': (str, False),
        'ComplianceResourceTypes': ([str], False),
        'TagKey': (str, False),
        'TagValue': (str, False),
    }


class SourceDetails(AWSProperty):
    props = {
        'EventSource': (str, True),
        'MaximumExecutionFrequency': (str, False),
        'MessageType': (str, True),
    }

    def validate(self):
        valid_freqs = [
            ONE_HOUR,
            THREE_HOURS,
            SIX_HOURS,
            TWELVE_HOURS,
            TWENTYFOUR_HOURS,
        ]
        freq = self.properties.get('MaximumExecutionFrequency')
        if freq and freq not in valid_freqs:
            raise ValueError(
                "MaximumExecutionFrequency (given: %s) must be one of: %s" % (
                    freq, ', '.join(valid_freqs)))


class Source(AWSProperty):
    props = {
        'Owner': (str, True),
        'SourceDetails': ([SourceDetails], False),
        'SourceIdentifier': (str, True),
    }


class ConfigRule(AWSObject):
    resource_type = "AWS::Config::ConfigRule"

    props = {
        'ConfigRuleName': (str, False),
        'Description': (str, False),
        'InputParameters': (dict, False),
        'MaximumExecutionFrequency': (str, False),
        'Scope': (Scope, False),
        'Source': (Source, True),
    }


class AggregationAuthorization(AWSObject):
    resource_type = "AWS::Config::AggregationAuthorization"

    props = {
        'AuthorizedAccountId': (str, True),
        'AuthorizedAwsRegion': (str, True),
    }


class OrganizationAggregationSource(AWSProperty):
    props = {
        'AllAwsRegions': (boolean, False),
        'AwsRegions': ([str], False),
        'RoleArn': (str, True),
    }


class AccountAggregationSources(AWSProperty):
    props = {
        'AccountIds': ([str], True),
        'AllAwsRegions': (boolean, False),
        'AwsRegions': ([str], False),
    }


class ConfigurationAggregator(AWSObject):
    resource_type = "AWS::Config::ConfigurationAggregator"

    props = {
        'AccountAggregationSources': ([AccountAggregationSources], False),
        'ConfigurationAggregatorName': (str, True),
        'OrganizationAggregationSource':
            (OrganizationAggregationSource, False),
    }


class RecordingGroup(AWSProperty):
    props = {
        'AllSupported': (boolean, False),
        'IncludeGlobalResourceTypes': (boolean, False),
        'ResourceTypes': ([str], False),
    }


class ConfigurationRecorder(AWSObject):
    resource_type = "AWS::Config::ConfigurationRecorder"

    props = {
        'Name': (str, False),
        'RecordingGroup': (RecordingGroup, False),
        'RoleARN': (str, True),
    }


class ConfigSnapshotDeliveryProperties(AWSProperty):
    props = {
        'DeliveryFrequency': (str, False),
    }


class DeliveryChannel(AWSObject):
    resource_type = "AWS::Config::DeliveryChannel"

    props = {
        'ConfigSnapshotDeliveryProperties':
            (ConfigSnapshotDeliveryProperties, False),
        'Name': (str, False),
        'S3BucketName': (str, True),
        'S3KeyPrefix': (str, False),
        'SnsTopicARN': (str, False),
    }


class OrganizationCustomRuleMetadata(AWSProperty):
    props = {
        'Description': (str, False),
        'InputParameters': (str, False),
        'LambdaFunctionArn': (str, True),
        'MaximumExecutionFrequency': (str, False),
        'OrganizationConfigRuleTriggerTypes': ([str], True),
        'ResourceIdScope': (str, False),
        'ResourceTypesScope': ([str], False),
        'TagKeyScope': (str, False),
        'TagValueScope': (str, False),
    }


class OrganizationManagedRuleMetadata(AWSProperty):
    props = {
        'Description': (str, False),
        'InputParameters': (str, False),
        'MaximumExecutionFrequency': (str, False),
        'ResourceIdScope': (str, False),
        'ResourceTypesScope': ([str], False),
        'RuleIdentifier': (str, True),
        'TagKeyScope': (str, False),
        'TagValueScope': (str, False),
    }


class OrganizationConfigRule(AWSObject):
    resource_type = "AWS::Config::OrganizationConfigRule"

    props = {
        'ExcludedAccounts': ([str], False),
        'OrganizationConfigRuleName': (str, True),
        'OrganizationCustomRuleMetadata':
            (OrganizationCustomRuleMetadata, False),
        'OrganizationManagedRuleMetadata':
            (OrganizationManagedRuleMetadata, False),
    }


class SsmControls(AWSProperty):
    props = {
        'ConcurrentExecutionRatePercentage': (integer, False),
        'ErrorPercentage': (integer, False),
    }


class ExecutionControls(AWSProperty):
    props = {
        'SsmControls': (SsmControls, False),
    }


class RemediationConfiguration(AWSObject):
    resource_type = "AWS::Config::RemediationConfiguration"

    props = {
        'Automatic': (boolean, False),
        'ConfigRuleName': (str, True),
        'ExecutionControls': (ExecutionControls, False),
        'MaximumAutomaticAttempts': (integer, False),
        'Parameters': (dict, False),
        'ResourceType': (str, False),
        'RetryAttemptSeconds': (integer, False),
        'TargetId': (str, True),
        'TargetType': (str, True),
        'TargetVersion': (str, False),
    }


class ConformancePackInputParameter(AWSProperty):
    props = {
        'ParameterName': (str, True),
        'ParameterValue': (str, True),
    }


class ConformancePack(AWSObject):
    resource_type = "AWS::Config::ConformancePack"

    props = {
        'ConformancePackInputParameters': ([ConformancePackInputParameter], False),  # NOQA
        'ConformancePackName': (str, True),
        'DeliveryS3Bucket': (str, True),
        'DeliveryS3KeyPrefix': (str, False),
        'TemplateBody': (str, False),
        'TemplateS3Uri': (str, False),
    }


class OrganizationConformancePack(AWSObject):
    resource_type = "AWS::Config::OrganizationConformancePack"

    props = {
        'ConformancePackInputParameters': ([ConformancePackInputParameter], False),  # NOQA
        'DeliveryS3Bucket': (str, True),
        'DeliveryS3KeyPrefix': (str, False),
        'ExcludedAccounts': ([str], False),
        'OrganizationConformancePackName': (str, True),
        'TemplateBody': (str, False),
        'TemplateS3Uri': (str, False),
    }



import re
from . import AWSObject, AWSProperty, Join, Tags
from .validators import boolean, integer, positive_integer

MEMORY_VALUES = [x for x in range(128, 3009, 64)]
RESERVED_ENVIRONMENT_VARIABLES = [
    'AWS_ACCESS_KEY',
    'AWS_ACCESS_KEY_ID',
    'AWS_DEFAULT_REGION',
    'AWS_EXECUTION_ENV',
    'AWS_LAMBDA_FUNCTION_MEMORY_SIZE',
    'AWS_LAMBDA_FUNCTION_NAME',
    'AWS_LAMBDA_FUNCTION_VERSION',
    'AWS_LAMBDA_LOG_GROUP_NAME',
    'AWS_LAMBDA_LOG_STREAM_NAME',
    'AWS_REGION',
    'AWS_SECRET_ACCESS_KEY',
    'AWS_SECRET_KEY',
    'AWS_SECURITY_TOKEN',
    'AWS_SESSION_TOKEN',
    'LAMBDA_RUNTIME_DIR',
    'LAMBDA_TASK_ROOT',
    'TZ'
]
ENVIRONMENT_VARIABLES_NAME_PATTERN = r'[a-zA-Z][a-zA-Z0-9_]+'


def validate_memory_size(memory_value):
    """ Validate memory size for Lambda Function
    :param memory_value: The memory size specified in the Function
    :return: The provided memory size if it is valid
    """
    memory_value = int(positive_integer(memory_value))
    if memory_value not in MEMORY_VALUES:
        raise ValueError("Lambda Function memory size must be one of:\n %s" %
                         ", ".join(str(mb) for mb in MEMORY_VALUES))
    return memory_value


def validate_variables_name(variables):
    for name in variables:
        if name in RESERVED_ENVIRONMENT_VARIABLES:
            raise ValueError("Lambda Function environment variables names"
                             " can't be none of:\n %s" %
                             ", ".join(RESERVED_ENVIRONMENT_VARIABLES))
        elif not re.match(ENVIRONMENT_VARIABLES_NAME_PATTERN, name):
            raise ValueError("Invalid environment variable name: %s" % name)

    return variables


class Code(AWSProperty):
    props = {
        'S3Bucket': (str, False),
        'S3Key': (str, False),
        'S3ObjectVersion': (str, False),
        'ZipFile': (str, False)
    }

    @staticmethod
    def check_zip_file(zip_file):
        maxlength = 4096
        toolong = (
            "ZipFile length cannot exceed %d characters. For larger "
            "source use S3Bucket/S3Key properties instead. "
            "Current length: %d"
        )

        if zip_file is None:
            return

        if isinstance(zip_file, str):
            z_length = len(zip_file)
            if z_length > maxlength:
                raise ValueError(toolong % (maxlength, z_length))
            return

        if isinstance(zip_file, Join):
            # This code tries to combine the length of all the strings in a
            # join. If a part is not a string, we do not count it (length 0).
            delimiter, values = zip_file.data['Fn::Join']

            # Return if there are no values to join
            if not values or len(values) <= 0:
                return

            # Get the length of the delimiter
            if isinstance(delimiter, str):
                d_length = len(delimiter)
            else:
                d_length = 0

            # Get the length of each value that will be joined
            v_lengths = [len(v) for v in values if isinstance(v, str)]

            # Add all the lengths together
            z_length = sum(v_lengths)
            z_length += (len(values)-1) * d_length

            if z_length > maxlength:
                raise ValueError(toolong % (maxlength, z_length))
            return

    def validate(self):
        zip_file = self.properties.get('ZipFile')
        s3_bucket = self.properties.get('S3Bucket')
        s3_key = self.properties.get('S3Key')
        s3_object_version = self.properties.get('S3ObjectVersion')

        if zip_file and s3_bucket:
            raise ValueError("You can't specify both 'S3Bucket' and 'ZipFile'")
        if zip_file and s3_key:
            raise ValueError("You can't specify both 'S3Key' and 'ZipFile'")
        if zip_file and s3_object_version:
            raise ValueError(
                "You can't specify both 'S3ObjectVersion' and 'ZipFile'"
            )
        Code.check_zip_file(zip_file)
        if not zip_file and not (s3_bucket and s3_key):
            raise ValueError(
                "You must specify a bucket location (both the 'S3Bucket' and "
                "'S3Key' properties) or the 'ZipFile' property"
            )


class VPCConfig(AWSProperty):

    props = {
        'SecurityGroupIds': (list, True),
        'SubnetIds': (list, True),
    }


class OnFailure(AWSProperty):
    props = {
        'Destination': (str, True),
    }


class DestinationConfig(AWSProperty):
    props = {
        'OnFailure': (OnFailure, True),
    }


class EventInvokeConfig(AWSObject):
    resource_type = "AWS::Lambda::EventInvokeConfig"

    props = {
        'DestinationConfig': (DestinationConfig, False),
        'FunctionName': (str, True),
        'MaximumEventAgeInSeconds': (integer, False),
        'MaximumRetryAttempts': (integer, False),
        'Qualifier': (str, True),
    }


class EventSourceMapping(AWSObject):
    resource_type = "AWS::Lambda::EventSourceMapping"

    props = {
        'BatchSize': (integer, False),
        'BisectBatchOnFunctionError': (boolean, False),
        'DestinationConfig': (DestinationConfig, False),
        'Enabled': (boolean, False),
        'EventSourceArn': (str, True),
        'FunctionName': (str, True),
        'MaximumBatchingWindowInSeconds': (integer, False),
        'MaximumRecordAgeInSeconds': (integer, False),
        'MaximumRetryAttempts': (integer, False),
        'ParallelizationFactor': (integer, False),
        'StartingPosition': (str, False),
    }


class DeadLetterConfig(AWSProperty):

    props = {
        'TargetArn': (str, False),
    }


class Environment(AWSProperty):

    props = {
        'Variables': (validate_variables_name, True),
    }


class TracingConfig(AWSProperty):

    props = {
        'Mode': (str, False),
    }


class Function(AWSObject):
    resource_type = "AWS::Lambda::Function"

    props = {
        'Code': (Code, True),
        'Description': (str, False),
        'DeadLetterConfig': (DeadLetterConfig, False),
        'Environment': (Environment, False),
        'FunctionName': (str, False),
        'Handler': (str, True),
        'KmsKeyArn': (str, False),
        'MemorySize': (validate_memory_size, False),
        'Layers': ([str], False),
        'ReservedConcurrentExecutions': (positive_integer, False),
        'Role': (str, True),
        'Runtime': (str, True),
        'Tags': (Tags, False),
        'Timeout': (positive_integer, False),
        'TracingConfig': (TracingConfig, False),
        'VpcConfig': (VPCConfig, False),
    }


class Permission(AWSObject):
    resource_type = "AWS::Lambda::Permission"

    props = {
        'Action': (str, True),
        'EventSourceToken': (str, False),
        'FunctionName': (str, True),
        'Principal': (str, True),
        'SourceAccount': (str, False),
        'SourceArn': (str, False),
    }


class VersionWeight(AWSProperty):

    props = {
        'FunctionVersion': (str, True),
        'FunctionWeight': (float, True),
    }


class AliasRoutingConfiguration(AWSProperty):

    props = {
        'AdditionalVersionWeights': ([VersionWeight], True),
    }


class ProvisionedConcurrencyConfiguration(AWSProperty):

    props = {
        'ProvisionedConcurrentExecutions': (integer, True),
    }


class Alias(AWSObject):
    resource_type = "AWS::Lambda::Alias"

    props = {
        'Description': (str, False),
        'FunctionName': (str, True),
        'FunctionVersion': (str, True),
        'Name': (str, True),
        'ProvisionedConcurrencyConfig':
            (ProvisionedConcurrencyConfiguration, False),
        'RoutingConfig': (AliasRoutingConfiguration, False),
    }


class Version(AWSObject):
    resource_type = "AWS::Lambda::Version"

    props = {
        'CodeSha256': (str, False),
        'Description': (str, False),
        'FunctionName': (str, True),
        'ProvisionedConcurrencyConfig':
            (ProvisionedConcurrencyConfiguration, False),
    }


class Content(AWSProperty):
    props = {
        'S3Bucket': (str, True),
        'S3Key': (str, True),
        'S3ObjectVersion': (str, False),
    }


class LayerVersion(AWSObject):
    resource_type = "AWS::Lambda::LayerVersion"

    props = {
        'CompatibleRuntimes': ([str], False),
        'Content': (Content, True),
        'Description': (str, False),
        'LayerName': (str, False),
        'LicenseInfo': (str, False),
    }


class LayerVersionPermission(AWSObject):
    resource_type = "AWS::Lambda::LayerVersionPermission"

    props = {
        'Action': (str, True),
        'LayerVersionArn': (str, True),
        'OrganizationId': (str, False),
        'Principal': (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSObject
from . import AWSProperty
from .validators import boolean
from .validators import integer


class CorsRule(AWSProperty):
    props = {
        'AllowedHeaders': ([str], False),
        'AllowedMethods': ([str], False),
        'AllowedOrigins': ([str], False),
        'ExposeHeaders': ([str], False),
        'MaxAgeSeconds': (integer, False),
    }


class Container(AWSObject):
    resource_type = "AWS::MediaStore::Container"

    props = {
        'AccessLoggingEnabled': (boolean, False),
        'ContainerName': (str, True),
        'CorsPolicy': ([CorsRule], False),
        'LifecyclePolicy': (str, False),
        'Policy': (str, False),
    }



from . import AWSObject, Tags, AWSProperty
from .validators import boolean


class DataResource(AWSProperty):
    props = {
        'Type': (str, True),
        'Values': ([str], False),
    }


class EventSelector(AWSProperty):
    props = {
        'DataResources': ([DataResource], False),
        'IncludeManagementEvents': (boolean, False),
        'ReadWriteType': (str, False),
    }


class Trail(AWSObject):
    resource_type = "AWS::CloudTrail::Trail"

    props = {
        'CloudWatchLogsLogGroupArn': (str, False),
        'CloudWatchLogsRoleArn': (str, False),
        'EnableLogFileValidation': (boolean, False),
        'EventSelectors': ([EventSelector], False),
        'IncludeGlobalServiceEvents': (boolean, False),
        'IsLogging': (boolean, True),
        'IsMultiRegionTrail': (boolean, False),
        'KMSKeyId': (str, False),
        'S3BucketName': (str, True),
        'S3KeyPrefix': (str, False),
        'SnsTopicName': (str, False),
        'Tags': (Tags, False),
        'TrailName': (str, False),
    }



# Copyright (c) 2012-2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, integer, waf_action_type


class Action(AWSProperty):
    props = {
        'Type': (waf_action_type, True)
    }


class FieldToMatch(AWSProperty):
    props = {
        'Data': (str, False),  # Conditional
        'Type': (str, True)
    }


class ByteMatchTuples(AWSProperty):
    props = {
        'FieldToMatch': (FieldToMatch, True),
        'PositionalConstraint': (str, True),
        'TargetString': (str, False),  # Conditional
        'TargetStringBase64': (str, False),  # Conditional
        'TextTransformation': (str, True)
    }


class IPSetDescriptors(AWSProperty):
    props = {
        'Type': (str, True),
        'Value': (str, True)
    }


class Predicates(AWSProperty):
    props = {
        'DataId': (str, True),
        'Negated': (boolean, True),
        'Type': (str, True)
    }


class GeoMatchConstraints(AWSProperty):
    props = {
        'Type': (str, True),
        'Value': (str, True)
    }


class Rules(AWSProperty):
    props = {
        'Action': (Action, True),
        'Priority': (integer, True),
        'RuleId': (str, True)
    }


class SqlInjectionMatchTuples(AWSProperty):
    props = {
        'FieldToMatch': (FieldToMatch, True),
        'TextTransformation': (str, True)
    }


class ByteMatchSet(AWSObject):
    resource_type = "AWS::WAFRegional::ByteMatchSet"

    props = {
        'ByteMatchTuples': ([ByteMatchTuples], False),
        'Name': (str, True)
    }


class IPSet(AWSObject):
    resource_type = "AWS::WAFRegional::IPSet"

    props = {
        'IPSetDescriptors': ([IPSetDescriptors], False),
        'Name': (str, True)
    }


class Rule(AWSObject):
    resource_type = "AWS::WAFRegional::Rule"

    props = {
        'MetricName': (str, True),
        'Name': (str, True),
        'Predicates': ([Predicates], False)
    }


class SqlInjectionMatchSet(AWSObject):
    resource_type = "AWS::WAFRegional::SqlInjectionMatchSet"

    props = {
        'Name': (str, True),
        'SqlInjectionMatchTuples': ([SqlInjectionMatchTuples], False)
    }


class WebACL(AWSObject):
    resource_type = "AWS::WAFRegional::WebACL"

    props = {
        'DefaultAction': (Action, True),
        'MetricName': (str, True),
        'Name': (str, True),
        'Rules': ([Rules], False)
    }


class WebACLAssociation(AWSObject):
    resource_type = "AWS::WAFRegional::WebACLAssociation"

    props = {
        'ResourceArn': (str, True),
        'WebACLId': (str, True),
    }


class SizeConstraint(AWSProperty):
    props = {
        'ComparisonOperator': (str, True),
        'FieldToMatch': (FieldToMatch, True),
        'Size': (integer, True),
        'TextTransformation': (str, True),
    }


class SizeConstraintSet(AWSObject):
    resource_type = "AWS::WAFRegional::SizeConstraintSet"

    props = {
        'Name': (str, True),
        'SizeConstraints': ([SizeConstraint], False),
    }


class XssMatchTuple(AWSProperty):
    props = {
        'FieldToMatch': (FieldToMatch, True),
        'TextTransformation': (str, True),
    }


class XssMatchSet(AWSObject):
    resource_type = "AWS::WAFRegional::XssMatchSet"

    props = {
        'Name': (str, True),
        'XssMatchTuples': ([XssMatchTuple], False),
    }


class RegexPatternSet(AWSObject):
    resource_type = "AWS::WAFRegional::RegexPatternSet"

    props = {
        'Name': (str, True),
        'RegexPatternStrings': ([str], True),
    }


class RateBasedRule(AWSObject):
    resource_type = "AWS::WAFRegional::RateBasedRule"

    props = {
        'MatchPredicates': ([Predicates], False),
        'MetricName': (str, True),
        'Name': (str, True),
        'RateKey': (str, True),
        'RateLimit': (integer, True),
    }


class GeoMatchSet(AWSObject):
    resource_type = "AWS::WAFRegional::GeoMatchSet"

    props = {
        'GeoMatchConstraints': ([GeoMatchConstraints], False),
        'Name': (str, True),
    }



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import (boolean, double, exactly_one, json_checker,
                         positive_integer, integer)


VALID_UNITS = ('Seconds', 'Microseconds', 'Milliseconds', 'Bytes', 'Kilobytes',
               'Megabytes', 'Gigabytes', 'Terabytes', 'Bits', 'Kilobits',
               'Megabits', 'Gigabits', 'Terabits', 'Percent', 'Count',
               'Bytes/Second', 'Kilobytes/Second', 'Megabytes/Second',
               'Gigabytes/Second', 'Terabytes/Second', 'Bits/Second',
               'Kilobits/Second', 'Megabits/Second', 'Gigabits/Second',
               'Terabits/Second', 'Count/Second', 'None')

VALID_TREAT_MISSING_DATA_TYPES = ('breaching', 'notBreaching', 'ignore',
                                  'missing')


def validate_unit(unit):
    """Validate Units"""

    if unit not in VALID_UNITS:
        raise ValueError("MetricStat Unit must be one of: %s" %
                         ", ".join(VALID_UNITS))
    return unit


def validate_treat_missing_data(value):
    """Validate TreatMissingData"""

    if value not in VALID_TREAT_MISSING_DATA_TYPES:
        raise ValueError("Alarm TreatMissingData must be one of: %s" %
                         ", ".join(VALID_TREAT_MISSING_DATA_TYPES))
    return value


class MetricDimension(AWSProperty):
    props = {
        'Name': (str, True),
        'Value': (str, True),
    }


class Metric(AWSProperty):
    props = {
        'Dimensions': ([MetricDimension], False),
        'MetricName': (str, False),
        'Namespace': (str, False),
    }


class MetricStat(AWSProperty):
    props = {
        'Metric': (Metric, True),
        'Period': (integer, True),
        'Stat': (str, True),
        'Unit': (validate_unit, False),
    }


class MetricDataQuery(AWSProperty):
    props = {
        'Expression': (str, False),
        'Id': (str, True),
        'Label': (str, False),
        'MetricStat': (MetricStat, False),
        'ReturnData': (boolean, False),
    }


class Alarm(AWSObject):
    resource_type = "AWS::CloudWatch::Alarm"

    props = {
        'ActionsEnabled': (boolean, False),
        'AlarmActions': ([str], False),
        'AlarmDescription': (str, False),
        'AlarmName': (str, False),
        'ComparisonOperator': (str, True),
        'DatapointsToAlarm': (positive_integer, False),
        'Dimensions': ([MetricDimension], False),
        'EvaluateLowSampleCountPercentile': (str, False),
        'EvaluationPeriods': (positive_integer, True),
        'ExtendedStatistic': (str, False),
        'InsufficientDataActions': ([str], False),
        'MetricName': (str, False),
        'Metrics': ([MetricDataQuery], False),
        'Namespace': (str, False),
        'OKActions': ([str], False),
        'Period': (positive_integer, False),
        'Statistic': (str, False),
        'Threshold': (double, False),
        'ThresholdMetricId': (str, False),
        'TreatMissingData': (validate_treat_missing_data, False),
        'Unit': (str, False),
    }

    def validate(self):
        conds = [
            'ExtendedStatistic',
            'Metrics',
            'Statistic',
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class Dashboard(AWSObject):
    resource_type = "AWS::CloudWatch::Dashboard"

    props = {
        'DashboardBody': ((str, dict), True),
        'DashboardName': (str, False),
    }

    def validate(self):
        name = 'DashboardBody'
        if name in self.properties:
            dashboard_body = self.properties.get(name)
            self.properties[name] = json_checker(dashboard_body)


class Range(AWSProperty):
    props = {
        'EndTime': (str, True),
        'StartTime': (str, True),
    }


class Configuration(AWSProperty):
    props = {
        'ExcludedTimeRanges': ([Range], False),
        'MetricTimeZone': (str, False),
    }


class AnomalyDetector(AWSObject):
    resource_type = "AWS::CloudWatch::AnomalyDetector"

    props = {
        'Configuration': (Configuration, False),
        'Dimensions': ([MetricDimension], False),
        'MetricName': (str, True),
        'Namespace': (str, True),
        'Stat': (str, True)
    }


class InsightRule(AWSObject):
    resource_type = "AWS::CloudWatch::InsightRule"

    props = {
        'RuleBody': (str, True),
        'RuleName': (str, True),
        'RuleState': (str, True),
    }



# Copyright (c) 2012-2017, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class Instance(AWSObject):
    resource_type = "AWS::ServiceDiscovery::Instance"

    props = {
        'InstanceAttributes': (dict, True),
        'InstanceId': (str, False),
        'ServiceId': (str, True),
    }


class PrivateDnsNamespace(AWSObject):
    resource_type = "AWS::ServiceDiscovery::PrivateDnsNamespace"

    props = {
        'Description': (str, False),
        'Name': (str, True),
        'Vpc': (str, True),
    }


class PublicDnsNamespace(AWSObject):
    resource_type = "AWS::ServiceDiscovery::PublicDnsNamespace"

    props = {
        'Description': (str, False),
        'Name': (str, True),
    }


class HealthCheckConfig(AWSProperty):
    props = {
        'FailureThreshold': (float, False),
        'ResourcePath': (str, False),
        'Type': (str, True),
    }


class HealthCheckCustomConfig(AWSProperty):
    props = {
        'FailureThreshold': (float, True)
    }


class DnsRecord(AWSProperty):
    props = {
        'TTL': (str, True),
        'Type': (str, True),
    }


class DnsConfig(AWSProperty):
    props = {
        'DnsRecords': ([DnsRecord], True),
        'NamespaceId': (str, False),
        'RoutingPolicy': (str, False),
    }


class Service(AWSObject):
    resource_type = "AWS::ServiceDiscovery::Service"

    props = {
        'Description': (str, False),
        'DnsConfig': (DnsConfig, False),
        'HealthCheckConfig': (HealthCheckConfig, False),
        'HealthCheckCustomConfig': (HealthCheckCustomConfig, False),
        'Name': (str, False),
        'NamespaceId': (str, False),
    }


class HttpNamespace(AWSObject):
    resource_type = "AWS::ServiceDiscovery::HttpNamespace"

    props = {
        'Description': (str, False),
        'Name': (str, True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import boolean


class Subscription(AWSProperty):
    props = {
        'Endpoint': (str, True),
        'Protocol': (str, True),
    }


class SubscriptionResource(AWSObject):
    resource_type = "AWS::SNS::Subscription"

    props = {
        'DeliveryPolicy': (dict, False),
        'Endpoint': (str, False),
        'FilterPolicy': (dict, False),
        'Protocol': (str, True),
        'RawMessageDelivery': (boolean, False),
        'RedrivePolicy': (dict, False),
        'Region': (str, False),
        'TopicArn': (str, True),
    }


class TopicPolicy(AWSObject):
    resource_type = "AWS::SNS::TopicPolicy"

    props = {
        'PolicyDocument': (policytypes, True),
        'Topics': (list, True),
    }


class Topic(AWSObject):
    resource_type = "AWS::SNS::Topic"

    props = {
        'DisplayName': (str, False),
        'KmsMasterKeyId': (str, False),
        'Subscription': ([Subscription], False),
        'Tags': (Tags, False),
        'TopicName': (str, False),
    }



# Copyright (c) 2012-2013, Antonio Alonso Dominguez <alonso.domin@gmail.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, AWSHelperFn, Tags
from .validators import (
    boolean, integer, positive_integer, double, defer
)


CHANGE_IN_CAPACITY = 'CHANGE_IN_CAPACITY'
PERCENT_CHANGE_IN_CAPACITY = 'PERCENT_CHANGE_IN_CAPACITY'
EXACT_CAPACITY = 'EXACT_CAPACITY'
ACTIONS_ON_FAILURE = ('TERMINATE_CLUSTER', 'CANCEL_AND_WAIT', 'CONTINUE',
                      'TERMINATE_JOB_FLOW')


def validate_action_on_failure(action_on_failure):
    """Validate action on failure for EMR StepConfig """

    if action_on_failure not in ACTIONS_ON_FAILURE:
        raise ValueError("StepConfig ActionOnFailure  must be one of: %s" %
                         ", ".join(ACTIONS_ON_FAILURE))
    return action_on_failure


class KeyValue(AWSProperty):
    props = {
        'Key': (str, True),
        'Value': (str, True)
    }

    def __init__(self, key=None, value=None, **kwargs):
        # provided for backward compatibility
        if key is not None:
            kwargs['Key'] = key
        if value is not None:
            kwargs['Value'] = value
        super(KeyValue, self).__init__(**kwargs)


MetricDimension = KeyValue


def additional_info_validator(xs):
    if not isinstance(xs, dict):
        raise ValueError("AdditionalInfo must be a dict of "
                         "string to string pairs")
    for k, v in xs.items():
        if not isinstance(k, str):
            raise ValueError('AdditionalInfo keys must be strings')
        if not isinstance(v, str):
            raise ValueError('AdditionalInfo values must be strings')

    return xs


class SecurityConfiguration(AWSObject):
    resource_type = "AWS::EMR::SecurityConfiguration"

    props = {
        'Name': (str, False),
        'SecurityConfiguration': (dict, True)
    }


class Application(AWSProperty):
    props = {
        'AdditionalInfo': (additional_info_validator, False),
        'Args': ([str], False),
        'Name': (str, False),
        'Version': (str, False)
    }


class ScriptBootstrapActionConfig(AWSProperty):
    props = {
        'Args': ([str], False),
        'Path': (str, True)
    }


class BootstrapActionConfig(AWSProperty):
    props = {
        'Name': (str, True),
        'ScriptBootstrapAction': (ScriptBootstrapActionConfig, True)
    }


def properties_validator(xs):
    if not isinstance(xs, dict):
        raise ValueError("ConfigurationProperties must be a dict of "
                         "string to string pairs")
    for k, v in xs.items():
        if not isinstance(k, str):
            raise ValueError('ConfigurationProperties keys must be strings')
        if not isinstance(v, str) and not isinstance(v, AWSHelperFn):
            raise ValueError('ConfigurationProperties values must be strings'
                             ' or helper functions')

    return xs


class Configuration(AWSProperty):
    props = {
        'Classification': (str, False),
        'ConfigurationProperties': (properties_validator, False)
    }


# we must define this one afterwards since Configuration does not exist
# before Configuration is done initializing
Configuration.props['Configurations'] = ([Configuration], False)


def market_validator(x):
    valid_values = ['ON_DEMAND', 'SPOT']
    if x not in valid_values:
        raise ValueError("Market must be one of: %s" %
                         ', '.join(valid_values))
    return x


def volume_type_validator(x):
    valid_values = ['standard', 'io1', 'gp2']
    if x not in valid_values:
        raise ValueError("VolumeType must be one of: %s" %
                         ', '.join(valid_values))
    return x


class VolumeSpecification(AWSProperty):
    props = {
        'Iops': (integer, False),
        'SizeInGB': (integer, True),
        'VolumeType': (volume_type_validator, True)
    }


class EbsBlockDeviceConfigs(AWSProperty):
    props = {
        'VolumeSpecification': (VolumeSpecification, True),
        'VolumesPerInstance': (integer, False)
    }


class EbsConfiguration(AWSProperty):
    props = {
        'EbsBlockDeviceConfigs': ([EbsBlockDeviceConfigs], False),
        'EbsOptimized': (boolean, False)
    }


class ScalingConstraints(AWSProperty):
    props = {
        'MinCapacity': (integer, True),
        'MaxCapacity': (integer, True)
    }


class CloudWatchAlarmDefinition(AWSProperty):
    props = {
        'ComparisonOperator': (str, True),
        'Dimensions': ([MetricDimension], False),
        'EvaluationPeriods': (positive_integer, False),
        'MetricName': (str, True),
        'Namespace': (str, False),
        'Period': (positive_integer, True),
        'Statistic': (str, False),
        'Threshold': (positive_integer, True),
        'Unit': (str, False),
    }


class ScalingTrigger(AWSProperty):
    props = {
        'CloudWatchAlarmDefinition': (CloudWatchAlarmDefinition, True),
    }


class SimpleScalingPolicyConfiguration(AWSProperty):
    props = {
        'AdjustmentType': (str, False),
        'CoolDown': (positive_integer, False),
        'ScalingAdjustment': (defer, True),
    }

    def validate(self):
        if 'AdjustmentType' in self.properties and \
           'ScalingAdjustment' in self.properties:

            valid_values = [
                CHANGE_IN_CAPACITY,
                PERCENT_CHANGE_IN_CAPACITY,
                EXACT_CAPACITY,
            ]

            adjustment_type = self.properties.get('AdjustmentType', None)
            scaling_adjustment = self.properties.get('ScalingAdjustment', None)

            if adjustment_type not in valid_values:
                raise ValueError(
                    'Only CHANGE_IN_CAPACITY, PERCENT_CHANGE_IN_CAPACITY, or'
                    ' EXACT_CAPACITY are valid AdjustmentTypes'
                )

            if adjustment_type == CHANGE_IN_CAPACITY:
                integer(scaling_adjustment)
            elif adjustment_type == PERCENT_CHANGE_IN_CAPACITY:
                double(scaling_adjustment)
                f = float(scaling_adjustment)
                if f < 0.0 or f > 1.0:
                    raise ValueError(
                        'ScalingAdjustment value must be between 0.0 and 1.0'
                        ' value was %0.2f' % f
                    )
            elif adjustment_type == EXACT_CAPACITY:
                positive_integer(scaling_adjustment)
            else:
                raise ValueError('ScalingAdjustment value must be'
                                 ' an integer or a float')


class ScalingAction(AWSProperty):
    props = {
        'Market': (market_validator, False),
        'SimpleScalingPolicyConfiguration': (
            SimpleScalingPolicyConfiguration, True
        )
    }


class ScalingRule(AWSProperty):
    props = {
        'Action': (ScalingAction, True),
        'Description': (str, False),
        'Name': (str, True),
        'Trigger': (ScalingTrigger, True),
    }


class AutoScalingPolicy(AWSProperty):
    props = {
        'Constraints': (ScalingConstraints, True),
        'Rules': ([ScalingRule], False),
    }


class InstanceGroupConfigProperty(AWSProperty):
    props = {
        'AutoScalingPolicy': (AutoScalingPolicy, False),
        'BidPrice': (str, False),
        'Configurations': ([Configuration], False),
        'EbsConfiguration': (EbsConfiguration, False),
        'InstanceCount': (positive_integer, True),
        'InstanceType': (str, True),
        'Market': (market_validator, False),
        'Name': (str, False),
    }


class SpotProvisioningSpecification(AWSProperty):
    props = {
        'BlockDurationMinutes': (positive_integer, False),
        'TimeoutAction': (str, True),
        'TimeoutDurationMinutes': (positive_integer, True),
    }


class InstanceFleetProvisioningSpecifications(AWSProperty):
    props = {
        'SpotSpecification': (SpotProvisioningSpecification, True),
    }


class InstanceTypeConfig(AWSProperty):
    props = {
        'BidPrice': (str, False),
        'BidPriceAsPercentageOfOnDemandPrice': (str, False),
        'Configurations': ([Configuration], False),
        'EbsConfiguration': (EbsConfiguration, False),
        'InstanceType': (str, True),
        'WeightedCapacity': (positive_integer, False),
    }


class InstanceFleetConfigProperty(AWSProperty):
    props = {
        'InstanceTypeConfigs': ([InstanceTypeConfig], False),
        'LaunchSpecifications':
            (InstanceFleetProvisioningSpecifications, False),
        'Name': (str, False),
        'TargetOnDemandCapacity': (positive_integer, False),
        'TargetSpotCapacity': (positive_integer, False),
    }


class PlacementType(AWSProperty):
    props = {
        'AvailabilityZone': (str, True)
    }


class JobFlowInstancesConfig(AWSProperty):
    props = {
        'AdditionalMasterSecurityGroups': ([str], False),
        'AdditionalSlaveSecurityGroups': ([str], False),
        'CoreInstanceFleet': (InstanceFleetConfigProperty, False),
        'CoreInstanceGroup': (InstanceGroupConfigProperty, False),
        'Ec2KeyName': (str, False),
        'Ec2SubnetId': (str, False),
        'Ec2SubnetIds': ([str], False),
        'EmrManagedMasterSecurityGroup': (str, False),
        'EmrManagedSlaveSecurityGroup': (str, False),
        'HadoopVersion': (str, False),
        'KeepJobFlowAliveWhenNoSteps': (boolean, False),
        'MasterInstanceFleet': (InstanceFleetConfigProperty, False),
        'MasterInstanceGroup': (InstanceGroupConfigProperty, False),
        'Placement': (PlacementType, False),
        'ServiceAccessSecurityGroup': (str, False),
        'TerminationProtected': (boolean, False)
    }


class KerberosAttributes(AWSProperty):
    props = {
        'ADDomainJoinPassword': (str, False),
        'ADDomainJoinUser': (str, False),
        'CrossRealmTrustPrincipalPassword': (str, False),
        'KdcAdminPassword': (str, True),
        'Realm': (str, True),
    }


class HadoopJarStepConfig(AWSProperty):
    props = {
        'Args': ([str], False),
        'Jar': (str, True),
        'MainClass': (str, False),
        'StepProperties': ([KeyValue], False)
    }


class StepConfig(AWSProperty):
    props = {
        'ActionOnFailure': (validate_action_on_failure, False),
        'HadoopJarStep': (HadoopJarStepConfig, True),
        'Name': (str, True),
    }


class Cluster(AWSObject):
    resource_type = "AWS::EMR::Cluster"

    props = {
        'AdditionalInfo': (dict, False),
        'Applications': ([Application], False),
        'AutoScalingRole': (str, False),
        'BootstrapActions': ([BootstrapActionConfig], False),
        'Configurations': ([Configuration], False),
        'CustomAmiId': (str, False),
        'EbsRootVolumeSize': (positive_integer, False),
        'Instances': (JobFlowInstancesConfig, True),
        'JobFlowRole': (str, True),
        'KerberosAttributes': (KerberosAttributes, False),
        'LogUri': (str, False),
        'Name': (str, True),
        'ReleaseLabel': (str, False),
        'ScaleDownBehavior': (str, False),
        'SecurityConfiguration': (str, False),
        'ServiceRole': (str, True),
        'Steps': ([StepConfig], False),
        'Tags': ((Tags, list), False),
        'VisibleToAllUsers': (boolean, False)
    }


class InstanceFleetConfig(AWSObject):
    resource_type = "AWS::EMR::InstanceFleetConfig"

    props = {
        'ClusterId': (str, True),
        'InstanceFleetType': (str, True),
        'InstanceTypeConfigs': ([InstanceTypeConfig], False),
        'LaunchSpecifications':
            (InstanceFleetProvisioningSpecifications, False),
        'Name': (str, False),
        'TargetOnDemandCapacity': (positive_integer, False),
        'TargetSpotCapacity': (positive_integer, False),
    }


class InstanceGroupConfig(AWSObject):
    resource_type = "AWS::EMR::InstanceGroupConfig"

    props = {
        'AutoScalingPolicy': (AutoScalingPolicy, False),
        'BidPrice': (str, False),
        'Configurations': ([Configuration], False),
        'EbsConfiguration': (EbsConfiguration, False),
        'InstanceCount': (integer, True),
        'InstanceRole': (str, True),
        'InstanceType': (str, True),
        'JobFlowId': (str, True),
        'Market': (market_validator, False),
        'Name': (str, False)
    }


def action_on_failure_validator(x):
    valid_values = ['CONTINUE', 'CANCEL_AND_WAIT']
    if x not in valid_values:
        raise ValueError("ActionOnFailure must be one of: %s" %
                         ', '.join(valid_values))
    return x


class Step(AWSObject):
    resource_type = "AWS::EMR::Step"

    props = {
        'ActionOnFailure': (action_on_failure_validator, True),
        'HadoopJarStep': (HadoopJarStepConfig, True),
        'JobFlowId': (str, True),
        'Name': (str, True)
    }
